function POLYACONSENSUS = find_polya_covsixmers(POLYAPAR)

%%%%%%%%%%q%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% find_polya_covsixmers.m
% author: Petra Philips
% June 2007
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This script goes through the list of putative polyA sites and
% searches for the most significant PAS (polyadenylation signals)
% which are variants of the AATAAA.
% It
% A. counts the histogramms of 6mers in a window [-300, -100] 
% relative to putative 3' end in order to compute  a background
% model.
% B. counts the histogramms of 6mers in the window [-100, 100] 
% around the putative 3' end of confidences in 'confidence'
% in order to check whether AATAAA or
% variants are occurring significantly more often. 
% For the significance the e-value for each sixmer is computed by
% taking the background model from A.
% C. The most significant 'aataaa' similar sixmer is chosen based
% on the e-value. All sequences containing this POS are removed.
%  Then the whole process is repeated iteratively until 99% of the
%  sequences are covered. 
% Results are saved in POLYACONSENSUS.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  wl = POLYAPAR.wl;
  maxpcCov = POLYAPAR.maxpcCov;
  cleave = POLYAPAR.cleavedata;
  signiflevel = 0.001;
  cdnatag = 0;%POLYAPAR.cdnatag;
  consensuslength = 6;%POLYAPAR.consensuslength;
  bpdist= 2;%POLYAPAR.bpdist;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%1%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

covlogfile = sprintf('%sconsensus_conf0_wl%i.log',POLYAPAR.logdir,POLYAPAR.wl);
fidlog = fopen(covlogfile,'wt');


figurename = sprintf('%sfigure_conf0_wl%i',POLYAPAR.logdir,POLYAPAR.wl);
resultsdataname = sprintf('%spolyA_covsixmers_conf0_wl%i.mat', POLYAPAR.logdir,POLYAPAR.wl);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set genomic info for loadin the sequences
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

genome_config = init_genome(POLYAPAR.fn_genome_config);
POLYACONSENSUS = {};

fprintf(fidlog, '\n Number of high conf cleave sites loaded: %d\n', length(cleave)) ;

% build data array for background model
clear BGDATA;
BGDATA = char(zeros(2*50*wl+1,length(cleave)));
FGDATA = char(zeros(2*50*wl+1,length(cleave)));

counter = 1;
for i=1:length(cleave)
   cleaveposition = cleave(i).pos;
   % cut out -300, -100 around putative polyA site
   if strcmp(cleave(i).strand,'+')
     bs= cleaveposition-100-2*50*wl;
     es = cleaveposition-100;
   else 
     bs = cleaveposition+100;
     es = cleaveposition+100+2*50*wl;
   end;
   
   clear backgroundsequ;
   if bs < 0 || es < 5,
     continue;
   end;
   
   backgroundsequ = lower(load_genomic(genome_config.contig_names{cleave(i).chr_num}, cleave(i).strand, ...
				 bs, es,genome_config));
   
   % build an array with all the background sequences
   BGDATA(:,i) = backgroundsequ;
   counter = counter + 1;
end;

noSequ= counter - 1;
BGDATA = BGDATA(:,1:noSequ);
if cdnatag
  BGDATA = BGDATA(1:50*wl,:);
end;

BGDATA = cutout(BGDATA, 'n');
% build an array with all the sixmers of the background sequences
BGXMER = acgt2intXmer(BGDATA,consensuslength);
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build data array for foreground model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
counterrepr = 0;
counter = 1;
cleave = POLYAPAR.cleavedata_highconf;

for i=1:length(cleave) 
  % if confidence is one of the input parameters take it for
  % foreground model
  if strcmp(cleave(i).strand, '+')
    strand = 1;
  else 
    strand = 0;
  end; 
  counterrepr = counterrepr +1;
   
  % cut out -100, +100 around putative polyA site
  % cut out -50*wl +50*wl around putative polyA site
  bs = cleave(i).pos-50*wl;
  es = cleave(i).pos+50*wl;

  clear sequ;
  sequ = lower(load_genomic(genome_config.contig_names{cleave(i).chr_num},cleave(i).strand, bs, es,genome_config));
  if es < 5,
    continue;
  end;
    
  FGDATA(:,counter) = sequ;
  counter = counter + 1;    
  
end; % for i=1:length(cleave)

noSequ= counter - 1;
FGDATA = FGDATA(:,1:noSequ);
if cdnatag
  FGDATA = FGDATA(1:50*wl,:);
end;

FGDATA = cutout(FGDATA, 'n');
% build an array with all the sixmers of the foregroundground sequences
FGXMER = acgt2intXmer(FGDATA,consensuslength);
noSitesInitial = size(FGXMER,2);
%save(BGandFGdataname, 'BGXMER', 'FGXMER');

counter_order_significance = 1;

fprintf(fidlog, 'Starting iteration for the significance model using a binomial test\n\n');

fprintf(fidlog, 'Order significance\t sixmer\t absobserved\t absexpected\t evalue\t position average +- std deviation\t normlike \n');


cov05 = 1;
pcCov = 0;
while pcCov <  maxpcCov
  
  fprintf(fidlog, 'Number of sequences in BG model: %d, \t FG model: %d\n', size(BGXMER,2), ...
		size(FGXMER,2));
  
  noSites = size(FGXMER,2);
  
% return the first 8 most significant 
  [xmer, absobserved, absexpected, evalue] ...
      = polya_evalue(BGXMER, FGXMER,8, bpdist);
  heurist = zeros(1,size(xmer,2));
  
  % if observed less than expected do not consider
  evalue(absobserved < absexpected) = 10^6;
  
 
  for cc = 1:size(xmer,2)   
    lvtmp = sum(FGXMER' == xmer(cc));
    locationVec = [];
    for i=1:size(FGXMER,1)
      locationVec= [locationVec i*ones(1,lvtmp(i))];
    end;
    [posmu(cc), possigma(cc)] = mynormfit(locationVec);
  end; % for cc
  
  fprintf(fidlog, '\n');
  % save only best one
  heurist = evalue .* exp(0.2*(abs(posmu-(wl*50 - 20)) - ...
			       min(abs(posmu-(wl*50 - 20)))) + 0.4*(possigma - min(possigma)));
  
  for cc = 1:size(xmer,2) 
    fprintf(fidlog, '\tOrd.signif. = %d\t %s \t absobs = %d \t absexp = %d \t evalue = %e\t pos.avg = %.1f +- %.1f\t heurist %g\n',counter_order_significance, int2acgt(xmer(cc),consensuslength), absobserved(cc), absexpected(cc), evalue(cc), posmu(cc), possigma(cc), heurist(cc));
  end;
  
  
   % do not continue if p-value is too high
   if min(evalue)*counter_order_significance > signiflevel
     fprintf(fidlog, 'Minimal evalue * %d less than %1.6f\n', counter_order_significance, signiflevel); 
    break;
  end;
  
  [bestmin besti]  = min(heurist);
  if bestmin == 0
    % heuristic == 0 is suspicious...if there are more take the one with most observed occurrences
    absobserveddummy = absobserved;
    absobserveddummy( heurist ~= 0) = 0;
    [bestmax besti] = max(absobserveddummy);
  end;
  
  lvtmp = sum(FGXMER' == xmer(besti));
  locationVec = [];
  for i=1:size(FGXMER,1)
    locationVec= [locationVec i*ones(1,lvtmp(i))];
  end;
    
  fprintf(fidlog, '**\tOrd.signif. = %d\t %s\t absobs = %d\t absexp = %d\t evalue = %e\t pos.avg. = %.1f +- %.1f\t heurist %g\n',counter_order_significance, int2acgt(xmer(besti),consensuslength), absobserved(besti), absexpected(besti), evalue(besti), posmu(besti), possigma(besti), heurist(besti));
   
  
  % cut out sequences containing xmer
 
  BGXMER = BGXMER(:, sum(BGXMER == xmer(besti)) == 0);
  FGXMER = FGXMER(:, sum(FGXMER == xmer(besti)) == 0);

  noSitesNew = size(FGXMER,2);
  pcSites = (noSites - noSitesNew)*100/noSitesInitial;
  pcCov = (noSitesInitial - noSitesNew)*100/noSitesInitial;
  
  COVERINGPOLYARESULT(counter_order_significance) = ...
      struct('sixmer', int2acgt(xmer(besti),consensuslength), 'absobserved', absobserved(besti), ...
	     'absexpected', absexpected(besti), 'evalue', evalue(besti), 'posmu', ...
	     posmu(besti), 'possigma', possigma(besti), 'locationVec', ...
	     locationVec);
  POLYACONSENSUS{counter_order_significance} = int2acgt(xmer(besti),consensuslength);
  fprintf(fidlog, '\t\t Covers %.2f of total sequences, \t total coverage until here %.2f\n\n', pcSites, pcCov);
  counter_order_significance =  counter_order_significance + 1;
  
end;
 
% print a nice figure
noprints = min(counter_order_significance -1,8);
  h= figure; hold on;
  for cc=1:noprints
    histvec = hist(COVERINGPOLYARESULT(cc).locationVec,wl*25+1);
    subplot(noprints, 1, cc); hist(COVERINGPOLYARESULT(cc).locationVec,wl*25+1); 
    axis([0 2*50*wl 0 max(histvec) ]); ylabel(COVERINGPOLYARESULT(cc).sixmer);
 end;

% xlabel(POLYAPAR.organism.name);
 
print( '-depsc2', [figurename '.eps']); 
 
save(resultsdataname,'COVERINGPOLYARESULT');
fprintf(fidlog, 'Results in %s\n\n', resultsdataname);
 
fclose(fidlog);
return;



