function ID_cnt = append_block_pred_to_file(block,fname,source,region_id,ID_cnt,model,regions)
%
%
% Assumes that file (fname) has been created, header has been added,
% and ID_cnt is properly initialized
%
% This can NOT handle multiple transcripts per gene (assumes unique segmentation)!
  
if fexist(fname)
%  fprintf('\nappending to file %s...\n', fname);
  [fd msg] = fopen(fname, 'a+');
  disp(msg);
else
  error('file %s does not exist!',fname);
end

strand = block.strand;
offset = block.offset;

if strand=='+',
  pred_start = block.block_start;
  pred_end   = block.block_end;
else
  assert(strand == '-');
  pred_start = block.block_end;
  pred_end   = block.block_start;
end


% to check for consensus
seq = [];
for r=1:length(regions),
  if equal(regions(r).chr, region_id) ...
        & regions(r).start<=pred_start & pred_end<=regions(r).stop,
    seq = regions(r).seq;
    seq_offset = regions(r).start-1;
    break
  end
end
assert(~isempty(seq));

% determine predicted genes and split segmenation
gene_pred = block.prediction.genes;

fprintf('  writing %i genes...\n', size(gene_pred,2));

for g=1:size(gene_pred,2),
  % create gene_id and mrna_id
  [gene_id ID_cnt] = new_gene_id(ID_cnt);
  % transform gene start and end into global coords  (closed intervals)
  if strand=='+',
    g_start = inf;
    g_stop  = -inf;
    for i=1:size(gene_pred,1),
      if isempty(gene_pred{i,g}),
        continue
      end

      g_start = min(g_start, offset+gene_pred{i,g}(1,1));
      g_stop  = max(g_stop,  offset+gene_pred{i,g}(end,2)-1);
    end
  else
    g_start = -inf;
    g_stop  = inf;
    for i=1:size(gene_pred,1),
      if isempty(gene_pred{i,g}),
        continue
      end
      g_start = max(g_start, offset-gene_pred{i,g}(end,2)+1);
      g_stop  = min(g_stop,  offset-gene_pred{i,g}(1,1));
    end
  end
  assert(~isinf(g_start) && ~isinf(g_stop));

  type  = 'gene';
  attr_str = ['ID=' gene_id];
  fprintf(fd,'%s\t%s\t%s\t%i\t%i\t.\t%s\t.\t%s\n', ...
          region_id,source,type,g_start,g_stop,strand,attr_str);

  for i=1:size(gene_pred,1),
    if isempty(gene_pred{i,g}),
      continue
    end

    [mrna_id ID_cnt] = new_mRNA_id(ID_cnt); 
    if strand=='+',
      t_start = offset+gene_pred{i,g}(1,1);
      t_stop  = offset+gene_pred{i,g}(end,2)-1;
    else
      t_start = offset-gene_pred{i,g}(end,2)+1;
      t_stop  = offset-gene_pred{i,g}(1,1);
    end
    type  = 'mRNA';
    attr_str = ['ID=' mrna_id ';Parent=' gene_id];
    fprintf(fd,'%s\t%s\t%s\t%i\t%i\t.\t%s\t.\t%s\n', ...
            region_id,source,type,t_start,t_stop,strand,attr_str);
  
    % find exons
    exon_types = [model.segments.utr5exon, ...
                  model.segments.transexon, ...
                  model.segments.cds_exon, ...
                  model.segments.utr3exon, ...
                  model.segments.polya_tail ...
                 ];
    exon_idx = find(ismember(gene_pred{i,g}(:,3),exon_types));
    exons = gene_pred{i,g}(exon_idx,:);
    intron_idx = find(gene_pred{i,g}(:,3)==model.segments.intron);
    introns = gene_pred{i,g}(intron_idx,1:2);

    % assert consensus at signal sites
    tis  = min(find(exons(:,3)==model.segments.cds_exon));
    stop = max(find(exons(:,3)==model.segments.cds_exon));
    if strand == '+'
      assert(equal(upper(seq(exons(tis,1)+offset-seq_offset:...
                             exons(tis,1)+2+offset-seq_offset)), 'ATG'));
      site_seq = upper(seq(exons(stop,2)+offset-seq_offset:...
                           exons(stop,2)+2+offset-seq_offset));
      assert(equal(site_seq,'TAA') || equal(site_seq,'TGA') || equal(site_seq,'TAG'));
      for j=1:size(introns,1),
        site_seq = upper(seq(introns(j,1)+offset-seq_offset:...
                             introns(j,1)+1+offset-seq_offset));
        assert(equal(site_seq, 'GT') || equal(site_seq, 'GC'));
        assert(equal(upper(seq(introns(j,2)-2+offset-seq_offset:...
                               introns(j,2)-1+offset-seq_offset)), 'AG'));
      end
    else
      assert(equal(upper(seq(offset-exons(tis,1)-2-seq_offset:...
                             offset-exons(tis,1)-seq_offset)), 'CAT'))
      site_seq = upper(seq(offset-exons(stop,2)-2-seq_offset:...
                           offset-exons(stop,2)-seq_offset));
      assert(equal(site_seq,'TTA') || equal(site_seq,'TCA') || equal(site_seq,'CTA'));
      for j=1:size(introns,1),
        site_seq = upper(seq(offset-introns(j,1)-1-seq_offset:...
                             offset-introns(j,1)-seq_offset));
        assert(equal(site_seq,'AC') || equal(site_seq,'GC'));
        assert(equal(upper(seq(offset-introns(j,2)+1-seq_offset:...
                               offset-introns(j,2)+2-seq_offset)), 'CT'));
      end    
    end
  
    % fuse 5' UTR to first cds exon
    tis_idx = min(find(exons(:,3)==model.segments.cds_exon));
    if ~isempty(tis_idx),
      if ~(exons(tis_idx,1) == exons(tis_idx-1,2)) keyboard; end
      exons(tis_idx,:) = [exons(tis_idx-1,1), ...
                          exons(tis_idx,2), ...
                          -1];
      exons(tis_idx-1,:) = [];
    end  

    % fuse polyA to 3' UTR
    polya_idx = min(find(exons(:,3)==model.segments.polya_tail));
    if ~isempty(polya_idx),
      exons(polya_idx-1,:) = [exons(polya_idx-1,1), ...
                              exons(polya_idx,2), ...
                              -1];
      exons(polya_idx,:) = [];    
    end
  
    % fuse 3' UTR to last cds exon
    stop_idx = max(find(exons(:,3)==model.segments.cds_exon));
    if ~isempty(stop_idx),
      assert(exons(stop_idx,2) == exons(stop_idx+1,1));
      exons(stop_idx,:) = [exons(stop_idx,1), ...
                          exons(stop_idx+1,2), ...
                          -1];
      exons(stop_idx+1,:) = [];    
    end
  
    % transform exons back into global coords (closed intervals)
    if strand=='+',
      exons = [offset+exons(:,1)  offset+exons(:,2)-1];
    else
      exons = [offset-exons(:,2)+1  offset-exons(:,1)]; 
      exons = exons(end:-1:1,:);
    end
 
    idx = 1:size(exons,1);
    if strand=='-', % reverse order
      idx = idx(end:-1:1);
    end
    for j=idx,
      [exon_id ID_cnt] = new_exon_id(ID_cnt);   
      type  = 'exon';
      attr_str = ['ID=' exon_id ';Parent=' mrna_id];
      fprintf(fd,'%s\t%s\t%s\t%i\t%i\t.\t%s\t.\t%s\n', ...
              region_id,source,type,exons(j,1),exons(j,2),strand,attr_str);
    end
  
    % find coding exons
    cds_idx = find(gene_pred{i,g}(:,3)==model.segments.cds_exon);
    cds_exons = gene_pred{i,g}(cds_idx,1:2);
    % include stop codon in cds
    cds_exons(end,2) = cds_exons(end,2)+3;

    % transform cds_exons back into global coords (closed intervals)
    if strand=='+',
      cds_exons = [offset+cds_exons(:,1)  offset+cds_exons(:,2)-1];
    else
      cds_exons = [offset-cds_exons(:,2)+1  offset-cds_exons(:,1)]; 
      cds_exons = cds_exons(end:-1:1,:);
    end
  
    carry = 0;
    idx = 1:size(cds_exons,1);
    if strand=='-', % reverse order and compute phase correctly
      idx = idx(end:-1:1);
    end
    for j=idx,
      [cds_id ID_cnt] = new_CDS_exon_id(ID_cnt);   
      type  = 'CDS';
      start = cds_exons(j,1);
      stop  = cds_exons(j,2);
      % transform carry (combined cds len modulo 3) to gff phase convention: 
      % #nucleotides to be removed from cds exon begin to reach first codon pos
      gff_phase = [];
      switch carry,
       case 0
        gff_phase = 0;
       case 1
        gff_phase = 2;
       case 2
        gff_phase = 1;
       otherwise
        error;
      end
      attr_str = ['ID=' cds_id ';Parent=' mrna_id];
      fprintf(fd,'%s\t%s\t%s\t%i\t%i\t.\t%s\t%i\t%s\n', ...
              region_id,source,type,start,stop,strand,gff_phase,attr_str);
      carry = mod(carry+(stop-start+1),3);
    end
    assert(carry==0);
  end
end

fclose(fd);


