function [num_obs,num_pred,num_correct,num_missg,num_wrong] = eval_segmentation(pred,truth,model,mask)
% function [num_obs,num_pred,num_correct,num_missg,num_wrong] = eval_segmentation(pred,truth,model,mask)
% evaluate on nucleotide, segment, and transcript (cds) level

assert(truth.segments(1,1)   == pred.segments(1,1));
assert(truth.segments(end,2) == pred.segments(end,2));

num_obs.segm = struct;
num_obs.nucl = struct;

num_pred.segm = struct;
num_pred.nucl = struct;

num_correct.segm = struct;
num_correct.nucl = struct;

num_missg.segm = struct;
num_wrong.segm = struct;

len = truth.segments(end,2);
segment_names = fieldnames(model.segments);

% select only segments within regions specified by mask
select_segm_idx = [];
masked_truth = [];
for i=1:size(mask,1),
  select_segm_idx = find((mask(i,1)<=truth.segments(:,1) ...
                           & truth.segments(:,1)<=mask(i,2)) ...
                          | (mask(i,1)<=truth.segments(:,2) ...
                           & truth.segments(:,2)<=mask(i,2)) ...
                          | (truth.segments(:,1)<=mask(i,1) ...
                           & mask(i,2)<=truth.segments(:,2)));
  
  tmp = truth.segments(select_segm_idx,:);
  if isempty(tmp), keyboard; end;
  if tmp(1,1) < mask(i,1),
    tmp(1,1) = mask(i,1);
    if size(tmp,1)>1 && tmp(1,1)==tmp(2,1),
      tmp(1,:) = [];
    end
    assert(tmp(1,2) > tmp(1,1));
  end
  if tmp(end,2) > mask(i,2),
    tmp(end,2) = mask(i,2);
    if size(tmp,1)>1 && tmp(end-1,2)==tmp(end,2),
      tmp(end,:) = [];
    end
    assert(tmp(end,1) < tmp(end,2));
  end
  
  masked_truth = [masked_truth; tmp];
  clear tmp
end

select_segm_idx = [];
masked_pred = [];
for i=1:size(mask,1),
  select_segm_idx = find((mask(i,1)<=pred.segments(:,1) ...
                          & pred.segments(:,1)<=mask(i,2)) ...
                         | (mask(i,1)<=pred.segments(:,2) ...
                          & pred.segments(:,2)<=mask(i,2)) ...
                         | (pred.segments(:,1)<=mask(i,1) ...
                          & mask(i,2)<=pred.segments(:,2)));
  tmp = pred.segments(select_segm_idx,:);
  if isempty(tmp), keyboard; end;
  if tmp(1,1) < mask(i,1),
    tmp(1,1) = mask(i,1);
    if size(tmp,1)>1 && tmp(1,1)==tmp(2,1),
      tmp(1,:) = [];
    end
    assert(tmp(1,2) > tmp(1,1));
  end
  if tmp(end,2) > mask(i,2),
    tmp(end,2) = mask(i,2);
    if size(tmp,1)>1 && tmp(end-1,2)==tmp(end,2),
      tmp(end,:) = [];
    end
    assert(tmp(end,1) < tmp(end,2));
  end
  
  masked_pred = [masked_pred; tmp];
  clear tmp  
end
if isempty(masked_truth) & isempty(masked_pred),
  truth.segments = zeros(0,3);
  pred.segments  = zeros(0,3);
else
  assert(~isempty(masked_truth) && ~isempty(masked_pred));
  truth.segments = masked_truth;
  pred.segments  = masked_pred;
  assert(truth.segments(1,1)   == pred.segments(1,1));
  assert(truth.segments(end,2) == pred.segments(end,2));
end
assert(size(truth.segments,2) == 3);
assert(size(pred.segments,2)  == 3);


% evaluate by segment type
for i=1:length(segment_names), 
  [obs_segms, pred_segms, correct_segms, correct_nucl] = ...
      eval_segment_type(truth.segments, pred.segments, segment_names{i}, model);
  
  num_obs.segm = setfield(num_obs.segm, segment_names{i}, size(obs_segms,1));
  num_obs.nucl = setfield(num_obs.nucl, segment_names{i}, sum(obs_segms(:,2)-obs_segms(:,1)));

  num_pred.segm = setfield(num_pred.segm, segment_names{i}, size(pred_segms,1));
  num_pred.nucl = setfield(num_pred.nucl, segment_names{i}, sum(pred_segms(:,2)-pred_segms(:,1)));

  num_correct.segm = setfield(num_correct.segm, segment_names{i}, size(correct_segms,1));
  num_correct.nucl = setfield(num_correct.nucl, segment_names{i}, sum(correct_nucl));

  missg = [];
  for j=1:size(obs_segms,1),
    missg(j) = ~any(correct_nucl(obs_segms(j,1):obs_segms(j,2)-1));
  end
  wrong = [];
  for j=1:size(pred_segms,1),
    wrong(j) = ~any(correct_nucl(pred_segms(j,1):pred_segms(j,2)-1));
  end
  
  num_missg.segm = setfield(num_missg.segm, segment_names{i}, sum(missg));
  num_wrong.segm = setfield(num_wrong.segm, segment_names{i}, sum(wrong));
end  

%%%%% TODO also determine missing genes and completely wrong gene predictions


% select only genes within regions specified by mask
select_gene_idx = [];
for i=1:length(truth.genes),
  select_gene_idx(i) = any(mask(:,1)<truth.genes{i}(1,1) ...
                           & truth.genes{i}(end,2)<mask(:,2));
end
truth.genes = truth.genes(find(select_gene_idx));

select_gene_idx = [];
for i=1:length(pred.genes),
  if isempty(pred.genes{i}),
    continue
  end  
  select_gene_idx(i) = any(mask(:,1)<pred.genes{i}(1,1) ...
                           & pred.genes{i}(end,2)<mask(:,2));
end
pred.genes = pred.genes(find(select_gene_idx));


num_obs.trsc = length(truth.genes);
num_pred.trsc = length(pred.genes);
num_correct.trsc = 0;

cds_type = model.segments.cds_exon;
for g=1:num_obs.trsc,
  % skip crippled genes
  if size(truth.genes{g},1)==1 & isnan(truth.genes{g}(1,3))
    continue
  end
  
  obs_start_idx = min(find(truth.genes{g}(:,3)==cds_type));
  obs_stop_idx  = max(find(truth.genes{g}(:,3)==cds_type));
  obs_cds = truth.genes{g}(obs_start_idx:obs_stop_idx,:);
  
  for h=1:num_pred.trsc,
    pred_start_idx = min(find(pred.genes{h}(:,3)==cds_type));
    pred_stop_idx  = max(find(pred.genes{h}(:,3)==cds_type));
    pred_cds = pred.genes{h}(pred_start_idx:pred_stop_idx,:);

    if equal(obs_cds, pred_cds),
      num_correct.trsc = num_correct.trsc + 1;
      break
    end
  end  
end

