function res = evaluate_block_predictions(blocks, verb, PAR, varargin);
% function evaluate_block_predictions(blocks, verb, varargin)
% to evaluate predictions in wormbase set source
%   source = 'twinscan'; % or 'genefinder'
% and use the following block file
%   pred_block_file = ['/fml/ag-raetsch/share/projects/nGASP/gene_finders/' source '_pred_blocks.mat'];

% Load blocks:
% Blocks must have a field prediction, a struct that resembles truth in
% that it has the fields segments. If prediction is a vector of
% several predictions, prediction(end) will be evaluated.
%load(pred_block_file, 'blocks');

addpath ../LSL/
addpath ../viz/

if ~exist('verb','var'),
  verb = 2;
end

% get the model used for LSL
PAR.dir = '/fml/ag-raetsch/share/projects/nGASP/';
PAR.bins     = 50;   % number of plif bins
PAR.use_gc   = 1; 
PAR.fn_polya_signals = [PAR.dir 'polya/RESULTS_FOR_2nd_LAYER/' ...
                    'signal_sixmers.mat'];
load(PAR.fn_polya_signals,'signal_sixmers');
PAR.polya_sixmers = {signal_sixmers.sixmer};
model = make_model(PAR);

% here we count the number of nucleotides in coding exons
nucleotides.num_obsv = 0;
nucleotides.num_pred = 0;
nucleotides.num_corr = 0;

% here we count coding exon entities
cds_exons.num_obsv = 0;
cds_exons.num_pred = 0;
cds_exons.num_corr = 0;
cds_exons.ME = 0; % missing exons (no overlap to pred. cds)
cds_exons.WE = 0; % wrong exons (no overlap to annotated cds)

% here we count whole transcripts (= genes, if not alt. spliced)
transcripts.num_obsv = 0;
transcripts.num_pred = 0;
transcripts.num_corr = 0;

% evaluation of predictions
for b=1:length(blocks),
  block = blocks(b);
  %block_len = abs(block.block_end-block.block_start)+1;
  block_len = length(block.genestr) ;
  
  % flag confirmed segments for separate evaluation
  if isempty(varargin),
    mask = [1 block_len];
  else
    assert(length(varargin)==1);
    if equal(varargin{1}, 'confirmed'),
      idx = find(block.gene_confirmed==1);
      mask = [block.gene_pos(idx,1)-500, ...
              block.gene_pos(idx,2)+500];
    elseif equal(varargin{1}, 'genic'),
      mask = [block.gene_pos(:,1)-500, ...
              block.gene_pos(:,2)+500];
    elseif equal(varargin{1}, 'unconfirmed'),
      idx = find(block.gene_confirmed==1);
      conf = [block.gene_pos(idx,1)-500, ...
              block.gene_pos(idx,2)+500];
      mask = [];
      if isempty(conf),
        mask = [1 block_len];
      else
        start = 1;
        for i=1:size(conf,1),
          mask = [mask; [start conf(i,1)-1]];
          start = conf(i,2)+1;
        end
        mask = [mask; start block_len];
      end
    elseif equal(varargin{1}, 'all'),
      mask = [1 block_len];
    else
      error('unknown evidence %s!', varagin{1});
    end
  end
  % join overlapping mask intervals
  mask_str = zeros(1,block_len);
  for i=1:size(mask,1),
    mask_str(mask(i,1):mask(i,2)) = 1;
  end

  sta_idx = find(mask_str==1 & [0 mask_str(1:end-1)]~=1);
  stp_idx = find(mask_str==1 & [mask_str(2:end)   0]~=1);
  clear mask 
  mask = [sta_idx' stp_idx'];
  
  num_obs     = [];
  num_pred    = [];
  num_correct = [];
  num_missg   = [];
  num_wrong   = [];
  [num_obs, num_pred, num_correct, num_missg, num_wrong] = ...
      eval_segmentation(block.prediction, block.truth(1), model, mask);

  % check all truth entries and take most favourable one
  for t=2:length(block.truth),
    [num_obs(t), num_pred(t), num_correct(t), num_missg(t), num_wrong(t)] = ...
        eval_segmentation(block.prediction, block.truth(t), model, mask);
  end
  best_t = 1;
  % use evaluation on exon level to decide which annotation to choose
  sn_best = num_correct(best_t).segm.cds_exon./(num_obs(best_t).segm.cds_exon+1e-8);
  sp_best = num_correct(best_t).segm.cds_exon./(num_pred(best_t).segm.cds_exon+1e-8);
  for t=2:length(block.truth),
    sn_new = num_correct(t).segm.cds_exon./(num_obs(t).segm.cds_exon+1e-8);
    sp_new = num_correct(t).segm.cds_exon./(num_pred(t).segm.cds_exon+1e-8);
    if sn_new+sp_new > sn_best+sp_best,
      best_t = t;
      sn_best = num_correct(best_t).segm.cds_exon./num_obs(best_t).segm.cds_exon;
      sp_best = num_correct(best_t).segm.cds_exon./num_pred(best_t).segm.cds_exon;
    end
  end
  
  %draw_pred(block.truth,block.prediction,model,mask)
  %keyboard
  
  % counts on nucleotide level
  nucleotides.num_obsv = nucleotides.num_obsv + num_obs(best_t).nucl.cds_exon;
  nucleotides.num_pred = nucleotides.num_pred + num_pred(best_t).nucl.cds_exon;
  nucleotides.num_corr = nucleotides.num_corr + num_correct(best_t).nucl.cds_exon;

  % counts on coding exon level (considers only exons of the
  % single most favourable transcript, not all exons in all transcripts)
  cds_exons.num_obsv = cds_exons.num_obsv + num_obs(best_t).segm.cds_exon;
  cds_exons.num_pred = cds_exons.num_pred + num_pred(best_t).segm.cds_exon;
  cds_exons.num_corr = cds_exons.num_corr + num_correct(best_t).segm.cds_exon;
  % missing exons (annotated exons with <1 bp overlap to predicted exons)
  cds_exons.ME = cds_exons.ME + num_missg(best_t).segm.cds_exon;
  % wrong exons (predicted exons with <1 bp overlap to annotated exons)
  cds_exons.WE = cds_exons.WE + num_wrong(best_t).segm.cds_exon;
  
  % counts on transcript level 
  transcripts.num_obsv = transcripts.num_obsv + num_obs(best_t).trsc;
  transcripts.num_pred = transcripts.num_pred + num_pred(best_t).trsc;
  transcripts.num_corr = transcripts.num_corr + num_correct(best_t).trsc;
  
  if verb >= 1,
    %cds_exons
    fprintf('  processed %i blocks\r',b);
  end
end
if verb >= 1,
  fprintf('\n');
end
if verb >= 2,
  fprintf('\n');
end

% Compute sensitivity and specificity
nucleotides.SN = nucleotides.num_corr/nucleotides.num_obsv;
nucleotides.SP = nucleotides.num_corr/nucleotides.num_pred;

cds_exons.SN = cds_exons.num_corr/cds_exons.num_obsv;
cds_exons.SP = cds_exons.num_corr/cds_exons.num_pred;
cds_exons.ME = cds_exons.ME./cds_exons.num_obsv;
cds_exons.WE = cds_exons.WE./cds_exons.num_pred;

transcripts.SN = transcripts.num_corr/transcripts.num_obsv;
transcripts.SP = transcripts.num_corr/transcripts.num_pred;

% Print evaluation summary
if verb >= 2,
  fprintf('Evaluation\n');
  fprintf('  on nucleotide level: (%2.2f%%)\n', mean([100*nucleotides.SN 100*nucleotides.SP]));
  fprintf('    sensitivity: %2.2f%%\n', 100*nucleotides.SN);
  fprintf('    specificity: %2.2f%%\n\n', 100*nucleotides.SP);
  
  fprintf('  on coding exon level: (%2.2f%%)\n', mean([100*cds_exons.SN 100*cds_exons.SP]));
  fprintf('    sensitivity: %2.2f%% (missing exons: %2.2f%%)\n', ...
	  100*cds_exons.SN, 100*cds_exons.ME);
  fprintf('    specificity: %2.2f%% (wrong exons:   %2.2f%%)\n\n', ...
	  100*cds_exons.SP, 100*cds_exons.WE);
  
  fprintf('  on transcript level: (%2.2f%%)\n',mean([100*transcripts.SN 100*transcripts.SP]));
  fprintf('    sensitivity: %2.2f%%\n', 100*transcripts.SN);
  fprintf('    specificity: %2.2f%%\n\n', 100*transcripts.SP);
end

% return results
res = [];
res.nucSn = 100 * nucleotides.SN;
res.nucSp = 100 * nucleotides.SP;
res.cexSn = 100 * cds_exons.SN;
res.cexSp = 100 * cds_exons.SP;
res.cexME = 100 * cds_exons.ME;
res.cexWE = 100 * cds_exons.WE;
res.rnaSn = 100 * transcripts.SN;
res.rnaSp = 100 * transcripts.SP;

