function [isoform_segments] = get_isoforms_eval(splicegraph,all_TIS,all_STOP,seq,model)
% [isoform_segments] = get_isoforms(splicegraph,all_TIS,all_STOP,seq,model)
% 
%
% Assumes that all input ccordinates are local and also 
% returns a segmentation in local coordinates
% 
% Generates all possible isoforms from the splicegraph,
% and locate all the start and stop codons.
% The reason for genesfilename is so that we can choose between the
% EST confirmed genes and the predicted genes.
%
% Written by: Cheng Soon Ong, 4 October 2006
% Modified on: 12 November 2006
% Modified on: 31v December 2006 (by Georg Zeller)  

addpath ../LSL/

vertices = splicegraph{1};
edges    = splicegraph{2};

for i=1:length(all_TIS),
  assert(equal(upper(seq(all_TIS(i):all_TIS(i)+2)), 'ATG'));
end
for i=1:length(all_STOP),
  assert(equal(upper(seq(all_STOP(i):all_STOP(i)+2)), 'TAA') ...
		 || equal(upper(seq(all_STOP(i):all_STOP(i)+2)), 'TAG') ...
		 || equal(upper(seq(all_STOP(i):all_STOP(i)+2)), 'TGA'));
end

% find all the leftmost and rightmost exons (transcription starts and stops)
tstart = [];
for ixv=1:size(vertices,2),
  if sum(edges(ixv,1:ixv-1)) == 0
    tstart(end+1) = ixv;
  end
end

% find all transcription ends
tstop = [];
for ixv=1:size(vertices,2),
  if sum(edges(ixv+1:end,ixv)) == 0
    tstop(end+1) = ixv;
  end
end

% do a depth first search through the graph, generating a list of isoforms
% - for each transcription start, do depth first search.
global isoforms
isoforms = {};

for ixt=tstart,
  dfs_visit(triu(edges),ixt,[]);
end

% for each path, find all the start and stop codons.
% and get the longest in frame coding sequence

tis=[]; stop=[]; len=[]; utr_len=[]; good_isoforms=[];
for ixf=1:length(isoforms),
  for i=1:length(all_TIS),
    for j=1:length(all_STOP),
      exons = vertices(:,isoforms{ixf})';
      idx = find(exons(:,1)<=all_TIS(i) & exons(:,2)>=all_TIS(i));
      if isempty(idx), 
		continue 
	  end
      utr_len_ = sum(exons(1:idx-1,2)-exons(1:idx-1,1)) ...
		             + all_TIS(i) - exons(idx,1);
      exons(1:idx-1,:) = [];
      exons(1,1) = all_TIS(i);
      idx = find(exons(:,1)<=all_STOP(j) & exons(:,2)>=all_STOP(j));
      if isempty(idx),
		continue 
	  end
      utr_len_ = utr_len_ + sum(exons(idx+1:end,2)-exons(idx+1:end,1)) ...
	             + exons(idx,2) - all_STOP(j);
      exons(idx+1:end,:) = [];
      exons(end,2) = all_STOP(j)+2;

      if ~mod(sum(exons(:,2)-exons(:,1)+1),3)
        str = [];
        for k=1:size(exons,1),
          str = [str seq(exons(k,1):exons(k,2))];
        end
        f = find_open_frames(lower(str),1);
        if f(1)==1,
          tis(end+1)           = all_TIS(i);
          stop(end+1)          = all_STOP(j);
          len(end+1)           = length(str);
          utr_len(end+1)       = utr_len_;
          good_isoforms{end+1} = isoforms{ixf};
        end
      end
    end
  end
end 

% convert good isoforms into segmentations
[tmp idx] = sort(len+1e-5*utr_len,'descend');

isoform_segments = {};

for i=1:length(idx)
  tis  = tis(idx);
  stop = stop(idx);

  exons = vertices(:,good_isoforms{idx(i)})';
  % convert to half-open segment intervals
  exons = [exons(:,1), ...
		   exons(:,2)+1, ...
		   repmat(model.segments.cds_exon, size(exons,1), 1)];
  % add introns
  introns = [];
  for j=2:size(exons,1)
	introns(j-1,:) = [exons(j-1,2), exons(j,1), model.segments.intron];
  end

  isoform_segments{i} = exons(1,:);
  for j=2:size(exons,1),
	isoform_segments{i} = [isoform_segments{i}; ...
						   introns(j-1,:);
						   exons(j,:)];
  end

  % insert TIS and STOP and fill 3rd col with segment types
  idx2 = find(isoform_segments{i}(:,1)<=tis(i) ...
			 & tis(i)<isoform_segments{i}(:,2));
  assert(length(idx2)==1);
  isoform_segments{i}(idx2+1:end+1,:) = isoform_segments{i}(idx2:end,:);
  isoform_segments{i}(idx2,2)   = tis(i);
  isoform_segments{i}(idx2+1,1) = tis(i);
  isoform_segments{i}(intersect(1:idx2, find(isoform_segments{i}(:,3)==model.segments.cds_exon)), 3) ...
	  = model.segments.utr5exon;

  idx2 = find(isoform_segments{i}(:,1)<=stop(i) ...
			 & stop(i)<isoform_segments{i}(:,2));
  assert(length(idx2)==1);
  isoform_segments{i}(idx2+1:end+1,:) = isoform_segments{i}(idx2:end,:);
  isoform_segments{i}(idx2,2)   = stop(i);
  isoform_segments{i}(idx2+1,1) = stop(i);
  isoform_segments{i}(intersect(idx2+1:end, find(isoform_segments{i}(:,3)==model.segments.cds_exon)),3) ...
	  = model.segments.utr3exon;


  % assert valid and tight segmentation
  assert(all(isoform_segments{i}(:,1)<isoform_segments{i}(:,2)));
  assert(all(isoform_segments{i}(1:end-1,2)==isoform_segments{i}(2:end,1)));
end
