function gen_cand_sites(PAR)

% gen_cand_sites(PAR)
% finds all candidate signal sites in sequences training_regions.seq and
% generates features XT. If PAR.fn_signal is given as input, true_signal
% sites are loaded and labels are accordingly assigned to each candidate. 
% 
% INPUT    PAR needs the following fields:
%	    Signal.name
%           fn_region 
%           fn_signal (if labels are to be assigned)
%           fn_cand (name of output file)
%           left_length (default 199)
%           right_length (default 199)
%	    use_cons
%                   
% OUTPUT   saves 'signal_cand' to fn_cand; includes fields: 
%          
%          'sig_pos'         position with signal consensus in
%                            genome coordinates
%          'sig_XT'	     genomic sequnece around potential signal site
%                            (left_length upstream and right_length downstream)
%          'sig_LT	     label : 1 for true signal site
%                            -1 for false signal site       
%

fprintf(1,'\ngen_cand_sites\n');

fid = fopen(sprintf('%s/gen_%s_sites.txt',PAR.FN.fn_log, PAR.Signal.name), 'a+');
fprintf(fid,'\n\n starting gen_cand_sites \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'loading fn_region: %s \n',PAR.fn_region);

if isfield(PAR,'fn_signal')
  load(PAR.fn_signal ,'training_signals');
  load(PAR.fn_region,'training_regions');
  regions = training_regions;
  fprintf(fid,'loading fn_signal: %s \n',PAR.fn_signal);
else
  load(PAR.fn_region,'test_regions');
  regions = test_regions;
end

if ~isfield(PAR,'left_length')|~isfield(PAR,'left_length')
  left_length = 199 ;
  right_length = 199 ;
else
  left_length = PAR.left_length;
  right_length = PAR.right_length; 
end
fprintf(fid,'left length: %i \n',left_length);
fprintf(fid,'right length: %i \n',right_length);
nofRegions = length(regions);

for r = 1:nofRegions
  strands = ['+','-'];
  for k = 1:length(strands) 
    chr   = regions(r).chr;
    start = regions(r).start;
    stop  = regions(r).stop;
    if PAR.use_cons
      cons_scores = regions(r).conservationScores ;
    else
      cons_scores = ones(size(regions(r).seq));
    end
    switch( strands(k) )
     case '+'  
      seq   = upper( regions(r).seq );
     case '-'
      seq   = upper(reverse_complement( regions(r).seq ));
      cons_scores = cons_scores(end:-1:1) ;
    end;
    
    %%% find all motiv sites
    pos = [];
    for c=1:length(PAR.Signal.consensus)
      pos = [pos findstr(seq(left_length+1:end-right_length),...
      		char(PAR.Signal.consensus(c)))+left_length];
    end 
    %%% get features (seq) for candidate sites
    sig_XT = char(zeros(length(pos),length(-left_length:right_length-1))) ; 
    sig_cons_scores = zeros(length(pos),length(-left_length:right_length-1)) ; 
    for i = 1:length(pos),
      sig_XT(i,:) = seq(pos(i)-left_length:pos(i)+right_length-1);
      sig_cons_scores(i,:) = cons_scores(pos(i)-left_length:pos(i)+right_length-1);
    end ;
    
    %%% transform to genome coordinates
    switch( strands(k) )
     case '+'  
        pos = start + (pos + 2) - 1;
     case '-'
      	pos = stop - (pos + 2)+1;
    end;
    
    %%% randomly choose SVM 
    rand('seed', r*12324) ;
    SVM = repmat([1:PAR.SETs.num_splits]',ceil(size(sig_XT,1)./PAR.SETs.num_splits),1);
    idx = randperm(size(sig_XT,1)) ;
    SVM = SVM(idx);
    
    
    %%% get lables  
    if isfield(PAR,'fn_signal')
      if r==2
        keyboard
      end
      
      true_pos = eval(sprintf('unique(training_signals(r).all_%s{k}(:,1))',PAR.Signal.name));
      sig_LT = -ones(length(pos),1) ; 
      [ii,ii1,ii2] = intersect(true_pos,pos);
      sig_LT(ii2) = 1; 
      fprintf(fid,'region %i strand %s:\t %d signals: (%3.1f %% positive)\n',...
              r,strands(k),length(pos),mean(sig_LT==1)*100);
      fprintf(fid,'\t number of invalid signals: %i (of %i)\n',...
      		length(true_pos)-sum(sig_LT==1),length(true_pos)) ;
      
      sig_cand.sig_LT{k} = sig_LT' ;
    else
      fprintf('region %i strand %s:/t %d acceptors, %d donors \n',...
              r,strands(k),length(apos),length(dpos));
    end    
    
    sig_cand.sig_pos{k} = pos ;
    sig_cand.sig_SVM{k} = SVM' ;
    sig_cand.sig_XT{k} = sig_XT' ;
    sig_cand.sig_cons_scores{k} = sig_cons_scores' ;
  end
  fprintf(fid,'saving sig_cand to file:  %s \n',[PAR.fn_cand,num2str(r),'.mat' ]);
  save([PAR.fn_cand,num2str(r),'.mat' ], 'sig_cand' );
end
% keyboard


