#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "common.h"
#include "genome.h"
#include "cdna2gff.h" // configuration options

int num_fixed = 0, num_total=0;
const int verbosity               = 1 ;
int num_intron_fixable = 0 ;
int num_intron_type1=0, num_intron_type2=0 ;

FILE * microexons_file = NULL ;

struct psl_line
{
	int blockcnt ;
	char strand ;
	char *Qname ;
	int contig_num ;    
	int covered ;
	int to_be_removed ;
	int *Qexonsizes, *Texonsizes, *Qstarts, *Qends, *Tstarts, *Tends, *intronsizes ;
} ;

struct psl_merged
{
	int blockcnt ;
	char strand ;
	char *Qname ;
	int contig_num ;    
	int *Tstarts, *Tends ;
	int *num_confirmed ;
	int *num_exon_confirmed ; // stores the number of times each exon has been (partially) confirmed
	int *num_intron_confirmed ; // stores the number of times each intron has been confirmed
	int valid ; 
	int to_be_removed ;
} ;

int psl_num = 0 ;
struct psl_line psls[PSL_SIZE] ;
int mpsl_num = 0 ;
struct psl_merged mpsls[PSL_SIZE] ;

/* convert a string list to an array */
void read_list(char *str, int *list, int targetlength)
{
	int i=0 ;
	char str2a[LIST_BUF_SIZE]="" ;
	char str2b[LIST_BUF_SIZE]="" ;
	strcpy(str2a,str) ;
	while (strlen(str2a)>0)
    {
		int item ;
		int ret=sscanf(str2a,"%i,%s", &item, str2b) ;
		assert(ret>0) ;
		strcpy(str2a,str2b) ;
		//fprintf(stdout, "%i,", item) ;
		list[i]=item ;
		i=i+1 ;
		if (ret!=2) break ;
		assert(i<LIST_SIZE) ;
    } ;
	assert(targetlength==i) ;
} ;


/* remove to short or too introns, gaps in the query sequence etc */
int check_psl(struct psl_line * psl, char strand)
{
	int i;
	
	fprintf(stdout, "check_psl: %s with %i blocks\n", psl->Qname, psl->blockcnt) ;
	
	// check for gaps in the alignment which are unplausible (short introns, Qgaps)
	for (i=0; i<psl->blockcnt-1; i++)
	{
		int Qgap = psl->Qstarts[i+1] - psl->Qstarts[i] - psl->Qexonsizes[i] ;
		int try_merge_exons = 0 ;
		
		if (psl->intronsizes[i]<0) 
		{
			fprintf(stdout,"### intron with negative size found (%i)\n", psl->intronsizes[i]) ;
			fprintf(stderr,"### intron with negative size found (%i)\n", psl->intronsizes[i]) ;
			return -1 ;
		} ;

		if ((Qgap==0) && 
			(psl->intronsizes[i] < MIN_INTRON_LEN) && 
			(psl->intronsizes[i]>0)) 
		{
			fprintf(stdout,"### found a too short intron (%i)\n", psl->intronsizes[i]) ;
			try_merge_exons = 1 ;
		} ;

		if ((Qgap>0) && (Qgap<=MAX_QGAP) && 
			(psl->intronsizes[i] < 2*MIN_INTRON_LEN+Qgap) && 
			(psl->intronsizes[i]>0)) 
		{
			fprintf(stdout,"### found a double gap -- assuming insert in cDNA (Qgap=%i, intron=%i)\n", Qgap, psl->intronsizes[i]) ;
			try_merge_exons = 1 ;
		} ;

		if ((Qgap>0) && 
			(psl->intronsizes[i] >= 2*MIN_INTRON_LEN+Qgap))
		{
			fprintf(stdout,"### found a double gap -- possibly a micro-exon -> don't fix it (Qgap=%i, intron=%i)\n", Qgap, psl->intronsizes[i]) ;
			//fprintf(microexons_file, "%s\t%s\t%c\t%i\t%i\t%i\t%i\n", psl->Qname, contig_names[psl->contig_num], psl->strand, 
			//psl->Tstarts[i], psl->Tends[i+1], Qgap, psl->intronsizes[i]) ;
			
		} ;

		if ((Qgap>0) && (Qgap<=MAX_QGAP) && (psl->intronsizes[i]==0)) 
		{
			fprintf(stdout,"### found a Qgap without Tgap (Qgap=%i)\n", Qgap) ;
			try_merge_exons = 1 ;
		} ;
		 
		if (try_merge_exons)
		{
			fprintf(stdout, "merging the two putative exons (assuming DNA is correct)\n") ;
			
			num_fixed++ ;
			int Qcorrection=psl->intronsizes[i]-Qgap ;

			psl->Texonsizes[i] = psl->intronsizes[i] + psl->Texonsizes[i] + psl->Texonsizes[i+1] ;
			psl->Qexonsizes[i] = Qgap + psl->Qexonsizes[i] + psl->Qexonsizes[i+1] + Qcorrection ;
			int j ;
			for (j=i; j<psl->blockcnt-1; j++)
			{
				psl->Qends[j]=psl->Qends[j+1] + Qcorrection ;
				if (strand=='+')
					psl->Tends[j]=psl->Tends[j+1] ;
				else
					psl->Tstarts[j]=psl->Tstarts[j+1] ;

				if (j<psl->blockcnt-2)
					psl->intronsizes[j]=psl->intronsizes[j+1] ;
				if (j>i)
				{
					psl->Qexonsizes[j]=psl->Qexonsizes[j+1] ;
					psl->Texonsizes[j]=psl->Texonsizes[j+1] ;
					if (strand=='+')
						psl->Tstarts[j]=psl->Tstarts[j+1] ;
					else
						psl->Tends[j]=psl->Tends[j+1] ;
					psl->Qstarts[j]=psl->Qstarts[j+1] + Qcorrection ;
				} ;
			} ;
			// the exon size will be correct for the DNA and incorrect for cDNA
			fprintf(stdout, "new: %s %i-%i[%i], %i-%i, intron=%i\n",  psl->Qname,
					psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
					psl->Qstarts[i+1], psl->Qends[i+1],
					psl->intronsizes[i]) ;
			i-- ;
			psl->blockcnt-- ;
		} ;
	} ;
	
	for (i=0; i<psl->blockcnt-1; i++)
	  {
		  int Qgap = psl->Qstarts[i+1] - psl->Qstarts[i] - psl->Qexonsizes[i] ;
		  
		  assert(Qgap>=0) ;
		  int to_be_fixed=0, is_fixed=0 ;
		  
		  fprintf(stdout, "%s %i-%i[%i], %i-%i, intron=%i Qgap=%i\n",  psl->Qname,
				  psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
				  psl->Qstarts[i+1], psl->Qends[i+1],
				  psl->intronsizes[i], Qgap) ;
		  if (Qgap<0) 
	      {
			  fprintf(stdout,"Qgap<0 (%i) -- bug in blat??\n", Qgap) ;
			  to_be_fixed = 1 ;
			  return -1 ;
	      } ;
		  if ((Qgap>0) && (psl->intronsizes[i] == 0)) 
	      {
			  fprintf(stdout,"found Qgap=%i without Tgap\n", Qgap) ;
			  to_be_fixed = 1 ;
	      } ;
		  if ((Qgap==0) && (psl->intronsizes[i] < MIN_INTRON_LEN) && (psl->intronsizes[i]>0)) 
	      {
			  fprintf(stdout,"found a too short intron (%i)\n", psl->intronsizes[i]) ;
			  to_be_fixed = 1 ;
			  return -1 ;
	      } ;
		  if (psl->intronsizes[i] > MAX_INTRON_LEN) 
	      {
			  fprintf(stdout,"found a too long intron (%i)\n", psl->intronsizes[i]) ;
			  to_be_fixed = 1 ;
			  return i+1 ;
	      } ;
		  if ((psl->intronsizes[i]!=0) && (Qgap!=0))
	      {
			  fprintf(stdout,"found double gap: Qgap=%i  Tgap=%i\n", Qgap, psl->intronsizes[i]) ;
			  to_be_fixed = 1 ;
	      } ;
		  int start_diff, end_diff, ok ;

		  if (psl->intronsizes[i] >= MIN_INTRON_LEN)
		  {
			  if (psl->strand=='+')
				  ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
									psl->Tstarts[i+1]-1, psl->strand,
									&start_diff, &end_diff) ;
			  else
				  ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
									psl->Tends[i+1]-1, psl->strand,
									&start_diff, &end_diff) ;
			  if (ok!=1)
			  {
				  fprintf(stdout,"found bad intron in %s[%i,%i,%i]: %i %i [%i]\n", 
						  psl->Qname, i, Qgap, psl->intronsizes[i], 
						  start_diff, end_diff, ok) ;
				  if (ok==-1)
				  {
					  num_intron_fixable++ ;
					  if (start_diff==end_diff)
					  {
						  if (psl->strand=='+')
						  {
							  psl->Tends[i]+=start_diff ;
							  psl->Tstarts[i+1]+=end_diff ;
						  } else
						  {
							  psl->Tends[i+1]-=start_diff ;
							  psl->Tstarts[i]-=end_diff ;
						  }
						  fprintf(stdout, "### intron ends shifted by %i\n", start_diff) ;
						  if (psl->strand=='+')
							  ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
												psl->Tstarts[i+1]-1, psl->strand,
												&start_diff, &end_diff) ;
						  else
							  ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
												psl->Tends[i+1]-1, psl->strand,
												&start_diff, &end_diff) ;
						  assert(ok==1) ;
					  }
				  }
				  if (ok!=1)
				  {
					  fprintf(stdout, "force a split\n") ;
					  to_be_fixed=1 ;
				  }
			  } ;
		  } ;
		  
		  if (((Qgap>0) && 
		       (Qgap <= MAX_QGAP_NOTGAP) && 
		       (psl->intronsizes[i] ==0)) ||
		      ((psl->intronsizes[i]>0) && 
		       (Qgap <= MAX_QGAP) && 
		       (psl->intronsizes[i] < GAP_TGAP_NOINTRON)))
		    {
		      is_fixed = 1 ;
			  num_fixed++ ;
			  fprintf(stdout," -> assuming DNA is correct and remove suspicios gaps (%i,%i):\n",
					  Qgap, psl->intronsizes[i]) ;
			  psl->Texonsizes[i] = psl->intronsizes[i] + psl->Qexonsizes[i] + psl->Qexonsizes[i+1] ;
			  psl->Qexonsizes[i] = Qgap + psl->Qexonsizes[i] + psl->Qexonsizes[i+1] ;
			  int j ;
			  for (j=i; j<psl->blockcnt-1; j++)
			  {
				  psl->Qends[j]=psl->Qends[j+1] ;
				  if (strand=='+')
					  psl->Tends[j]=psl->Tends[j+1] ;
				  else
					  psl->Tstarts[j]=psl->Tstarts[j+1] ;
				  if (j<psl->blockcnt-2)
					  psl->intronsizes[j]=psl->intronsizes[j+1] ;
				  if (j>i)
				  {
					  psl->Qexonsizes[j]=psl->Qexonsizes[j+1] ;
					  psl->Texonsizes[j]=psl->Texonsizes[j+1] ;
					  if (strand=='+')
						  psl->Tstarts[j]=psl->Tstarts[j+1] ;
					  else
						  psl->Tends[j]=psl->Tends[j+1] ;
					  psl->Qstarts[j]=psl->Qstarts[j+1] ;
				  } ;
			  } ;
			  // the exon size will be correct for the DNA and incorrect for cDNA
			  fprintf(stdout, "new: %s %i-%i[%i], %i-%i, intron=%i\n",  psl->Qname,
					  psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
					  psl->Qstarts[i+1], psl->Qends[i+1],
					  psl->intronsizes[i]) ;
			  i-- ;
			  psl->blockcnt-- ;
			} ;
		  if (to_be_fixed!=is_fixed) 
	      {
			  fprintf(stdout, "NOT FIXED tof=%i,if=%i\n",to_be_fixed,is_fixed) ;
			  fprintf(stdout, "%s %i-%i[%i], %i-%i, intron=%i Qgap=%i\n",  psl->Qname,
					  psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
					  psl->Qstarts[i+1], psl->Qends[i+1],
					  psl->intronsizes[i], Qgap) ;
			  return i+1 ;
	      } ;
	  } ;

	return 0 ;
} ;

/* free a psl structure */
void free_psl(struct psl_line *psl) 
{
  free(psl->Qname) ;
  free(psl->Qexonsizes) ;
  free(psl->Texonsizes) ;
  free(psl->Qstarts) ;
  free(psl->Qends) ;
  free(psl->Tstarts) ;
  free(psl->Tends) ;
  free(psl->intronsizes) ;
} 

void free_all_psl()
{
  int i=0 ;
  for (i=0; i<psl_num; i++)
    free_psl(&psls[i]) ;
} 

void free_mpsl(struct psl_merged *psl) 
{
  free(psl->Qname) ;
  free(psl->Tstarts) ;
  free(psl->Tends) ;
  free(psl->num_intron_confirmed) ;
  free(psl->num_exon_confirmed) ;
} 
 
void free_all_mpsl()
{
  int i=0 ;
  for (i=0; i<mpsl_num; i++)
    free_mpsl(&mpsls[i]) ;
} 

/* copy a psl structure */
void copy_psl(struct psl_line*dst, struct psl_line *src, int start)
{
  assert(start>=0) ;
  dst->blockcnt=src->blockcnt-start ; 
  dst->strand=src->strand ;
  dst->Qname=(char*)malloc(strlen(src->Qname)+2) ; 
  sprintf(dst->Qname, "%s*", src->Qname) ;
  dst->contig_num=src->contig_num ;
  dst->Qexonsizes=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qexonsizes, &src->Qexonsizes[start], src->blockcnt-start) ;
  dst->Texonsizes=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Texonsizes, &src->Texonsizes[start], src->blockcnt-start) ;
  dst->Qstarts=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qstarts, &src->Qstarts[start], src->blockcnt-start) ;
  dst->Qends=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qends, &src->Qends[start], src->blockcnt-start) ;
  dst->Tstarts=(int *)calloc((src->blockcnt-start),sizeof(int)) ;
  copy_int(dst->Tstarts, &src->Tstarts[start], src->blockcnt-start) ;
  dst->Tends=(int *)calloc((src->blockcnt-start),sizeof(int)) ;
  copy_int(dst->Tends, &src->Tends[start], src->blockcnt-start) ;
  dst->intronsizes=(int *)malloc(((src->blockcnt-start)-1)*sizeof(int)) ;
  copy_int(dst->intronsizes, &src->intronsizes[start], (src->blockcnt-start)-1) ;
} ;

void reverse_array(int * ar, int length)
{
  int * buf = (int *)malloc(sizeof(int)*length) ;
  int i ;
  for (i=0; i<length; i++)
    buf[length-i-1]=ar[i] ;
  copy_int(ar, buf, length) ;
  free(buf) ;
} ;

void read_psl(FILE *psl_file, int target_contig_num, char target_strand)
{
  int num_splitted = 0, num_not_taken=0, num_rejected=0 ;
  char line_buf[LINE_BUF_SIZE] ;
  int match, mismatch, repmatch, Ns, Qgapcnt, Qgapbs, Tgapcnt, Tgapbs;
  int Qsize, Qstart, Qend, Tsize, Tstart, Tend, blockcnt ;

  char strand ;
  char Qname[NAME_BUF_SIZE], Tname[NAME_BUF_SIZE], 
    exonsizes_str[LIST_BUF_SIZE], Qstarts_str[LIST_BUF_SIZE], Tstarts_str[LIST_BUF_SIZE] ;
  int exonsizes[LIST_SIZE], Qstarts[LIST_SIZE], Tstarts[LIST_SIZE] ;
  int line_num = 0 ;
  psl_num=0 ;

  char prevQname[NAME_BUF_SIZE]="" ;
  char iQname[NAME_BUF_SIZE]="" ;
  int iQ=0 ;

  while (1)
  {
      read_line(psl_file, line_buf, LINE_BUF_SIZE) ;
      if (strlen(line_buf)==0)
		  break ;
      sscanf(line_buf, "%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%c\t%s\t"
			 "%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i\t%s\t%s\t%s", 
			 &match, &mismatch, &repmatch, &Ns, &Qgapcnt, &Qgapbs, &Tgapcnt, &Tgapbs,
			 &strand, Qname, &Qsize, &Qstart, &Qend, Tname, &Tsize, &Tstart, &Tend, 
			 &blockcnt,exonsizes_str,Qstarts_str,Tstarts_str) ;
	  
      if ((target_strand==strand) && 
		  (strcmp(contig_names[target_contig_num],Tname)==0))
	  {
		  int start_missing = Qstart ;
		  int end_missing = Qsize - Qend ;
		  int missing = start_missing + end_missing ;
		  
		  if ((match+missing < MIN_CDNA_COVER_FRAC*Qsize) || 
			  (end_missing > MAX_END_MISSING) || (start_missing> MAX_START_MISSING))
		  {
			  num_not_taken+=blockcnt ;
		  }
		  else
		  {
			  if (strcmp(Qname,prevQname)==0)
			  {
				  sprintf(iQname, "%s-%c%c", Qname, 65+(iQ/26),65+(iQ%26)) ;
				  iQ++ ;
				  assert(iQ<26*26) ;
			  } else
			  {
				  sprintf(iQname, "%s", Qname) ;
				  iQ=0 ;
			  } ;
			  //fprintf(stdout, "ok: %s %s XXXXX\n", iQname, prevQname) ;
			  strcpy(prevQname,Qname) ;

			  num_total+=blockcnt ;
			  read_list(exonsizes_str, exonsizes,blockcnt) ;
			  read_list(Qstarts_str, Qstarts,blockcnt) ;
			  read_list(Tstarts_str, Tstarts,blockcnt) ;
			  psls[psl_num].covered = 0 ;
			  if ((end_missing > MAX_END_MISSING2) || (start_missing> MAX_START_MISSING2))
			  {
				  fprintf(stdout, "start_missing = %i   end_missing = %i -> to_be_removed=1\n", start_missing, end_missing) ;
				  psls[psl_num].to_be_removed = 1 ;
			  }
			  else
				  psls[psl_num].to_be_removed = 0 ;
			  psls[psl_num].blockcnt=blockcnt ; psls[psl_num].strand=strand ;
			  psls[psl_num].Qname=strdup(iQname) ; psls[psl_num].contig_num=target_contig_num ;    
			  psls[psl_num].Qexonsizes=(int *)malloc(blockcnt*sizeof(int)) ;
			  psls[psl_num].Texonsizes=(int *)malloc(blockcnt*sizeof(int)) ;
			  copy_int(psls[psl_num].Qexonsizes,exonsizes, blockcnt) ;
			  copy_int(psls[psl_num].Texonsizes,exonsizes, blockcnt) ;
			  psls[psl_num].Qstarts=(int *)malloc(blockcnt*sizeof(int)) ;
			  psls[psl_num].Qends=(int *)malloc(blockcnt*sizeof(int)) ;
			  copy_int(psls[psl_num].Qstarts,Qstarts, blockcnt) ;
			  psls[psl_num].Tstarts=(int *)calloc(blockcnt,sizeof(int)) ;
			  copy_int(psls[psl_num].Tstarts,Tstarts, blockcnt) ;
			  psls[psl_num].Tends=(int *)calloc(blockcnt,sizeof(int)) ;
			  psls[psl_num].intronsizes=(int *)malloc((blockcnt-1)*sizeof(int)) ;
			  
			  if (strand=='-')
			  {
				  int * buf = psls[psl_num].Tstarts ;
				  psls[psl_num].Tstarts=psls[psl_num].Tends ;
				  psls[psl_num].Tends=buf ;
			  } ;
			  int i;
			  for (i=0; i<psls[psl_num].blockcnt; i++)
			  {
				  psls[psl_num].Qends[i] = psls[psl_num].Qstarts[i] + psls[psl_num].Qexonsizes[i]-1 ;
				  if (strand=='+')
				  {
					  psls[psl_num].Tstarts[i]++ ;
					  psls[psl_num].Tends[i] = psls[psl_num].Tstarts[i] + psls[psl_num].Texonsizes[i]-1 ;
				  } 
				  else
				  {
					  psls[psl_num].Tends[i]++ ;
					  psls[psl_num].Tstarts[i] = psls[psl_num].Tends[i] + psls[psl_num].Texonsizes[i]-1 ;
				  } ;
				  
				  assert(psls[psl_num].Qexonsizes[i]>=0) ;
				  assert(psls[psl_num].Texonsizes[i]>=0) ;
			  }
			  for (i=0; i<psls[psl_num].blockcnt-1; i++)
			  {
				  if (strand=='+')
					  psls[psl_num].intronsizes[i]=psls[psl_num].Tstarts[i+1]-psls[psl_num].Tends[i]-1 ;
				  else
					  psls[psl_num].intronsizes[i]=psls[psl_num].Tends[i+1]-psls[psl_num].Tstarts[i]-1 ;
				  assert(psls[psl_num].intronsizes[i]>=0) ;
				  //fprintf(stdout,"introns=%i\n", psls[psl_num].intronsizes[i]) ;
				  //		assert(psls[psl_num].intronsizes[i]<200000) ;
			  } ;
			  
			  for (i=0; i<psls[psl_num].blockcnt; i++)
				fprintf(stdout, "b %i: %s %i-%i  %i-%i\n",  i, psls[psl_num].Qname, Qstarts[i], psls[psl_num].Qends[i], psls[psl_num].Tstarts[i], psls[psl_num].Tends[i]) ;
			  
			  int not_ok_idx=1 ;
			  while (not_ok_idx>0)
			  {
				  fprintf(stdout, "psl=%i\n", psl_num) ;
				  
				  not_ok_idx=check_psl(&(psls[psl_num]),strand) ;
				  if (not_ok_idx>0)
				  {
					  fprintf(stdout,"not_ok_idx=%i\n", not_ok_idx) ;
					  copy_psl(&psls[psl_num+1], &psls[psl_num], not_ok_idx) ;
					  psls[psl_num].blockcnt=not_ok_idx ;
					  psls[psl_num].to_be_removed=1 ;
					  psls[psl_num+1].to_be_removed=1 ;
				  } ;
				  
				  if (strand=='-')
				  {
					  reverse_array(psls[psl_num].Tstarts, psls[psl_num].blockcnt) ;
					  reverse_array(psls[psl_num].Tends, psls[psl_num].blockcnt) ;
					  reverse_array(psls[psl_num].Texonsizes, psls[psl_num].blockcnt) ;
					  reverse_array(psls[psl_num].intronsizes, psls[psl_num].blockcnt-1) ;
				  } ;
				  
				  if (not_ok_idx>=0)
				  {
					  if (not_ok_idx>0) 
					  {
						  num_splitted++ ;
						  fprintf(stdout, "dosplit (%i,%i,%i)\n", not_ok_idx, blockcnt, psls[psl_num].blockcnt) ;
					  }
					  else
						  if (blockcnt!=psls[psl_num].blockcnt){
							  
							  num_fixed++ ;
							  fprintf(stdout, "psl was fixed (%i,%i)\n", blockcnt, psls[psl_num].blockcnt) ;
						  }
					  psl_num=psl_num+1 ;
					  assert(psl_num<PSL_SIZE) ;
				  } 
				  else
				  {
					  num_rejected++ ;
					  free_psl(&psls[psl_num]) ;
				  };
			  } ;
		  } ;
	  } ;
      line_num++ ;
  } ;
  fprintf(stdout,"accepted %i psl lines, (%i taken, %i fixed, %i splitted, %i intron_fixable, %i rejected, %i not taken)\n", psl_num, num_total, num_fixed, num_splitted, num_intron_fixable, num_rejected, num_not_taken) ;
} ;

int check_mismatch(int num1, struct psl_line * psl) 
{
  int i,j=0 ;
  int pos1, pos2 ;
  int found = 0 ;
  
  for (i=0; i < psls[num1].blockcnt; i++) 
  {
      for (j=0; j < psl->blockcnt; j++)
	  {
		  if ((psls[num1].Tstarts[i] == psl->Tstarts[j]) ||
			  (psls[num1].Tends[i] == psl->Tends[j]))
		  {
			  found = 1 ;
			  break ;
		  } ;
	  } ;
      if (found)
		  break ;
  } ;
  if (!found)
	  return -1 ; // no overlap at all
  
  pos1=i ;
  pos2=j ;
  i=0 ;
  while ((pos1+i<psls[num1].blockcnt) && (pos2+i<psl->blockcnt))
  {
      if ((psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i]) ||
		  (psls[num1].Tends[pos1+i]!=psl->Tends[pos2+i]))
		  return 1 ;
      i++ ;
  } ;
  
  i=1 ;
  while ((pos1-i>=0) && (pos2-i>=0))
  {
      if ((psls[num1].Tstarts[pos1-i]!=psl->Tstarts[pos2-i]) ||
		  (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i]))
		  return 1 ;
      i++ ;
  } ;
  
  if (((psl->strand=='+') && (pos1==pos2) && (psls[num1].blockcnt!=psl->blockcnt))||
	  ((psl->strand=='-') && (psls[num1].blockcnt-pos1==psl->blockcnt-pos2) && (psls[num1].blockcnt!=psl->blockcnt)))
  {
	  //fprintf(stdout, "### same starting exons but different stop in genes %s %s\nAssuming incomplete cDNA (taking the longer one)\n",psls[num1].Qname, psl->Qname) ;
	  return 1 ;
  }

  if (((psl->strand=='+') && 
	   (psls[num1].blockcnt-pos1==psl->blockcnt-pos2) && 
	   (psls[num1].blockcnt!=psl->blockcnt)) ||
	  ((psl->strand=='-') && 
	   (pos1==pos2) && 
	   (psls[num1].blockcnt!=psl->blockcnt)))
  {
	  //fprintf(stdout, "### same ending exons but different start in genes %s %s\nAssuming alternative TIS\n",psls[num1].Qname, psl->Qname) ;
	  return 1 ;
  }

  return 0 ;
} 

/* finds a list of psls, that match the current psl (overlap, but no mismatch) */
int find_match(struct psl_line * psl, int * found) 
{
	int i,j,k ;
	int num_match = 0 ;

	for (i=0; i<psl_num; i++)
    {
		int match = 0 ;
		for (j=0; j<psls[i].blockcnt; j++)
		{
			for (k=0; k<psl->blockcnt; k++)
			{
				// exact exon matches
				if ((psl->Tstarts[k]==psls[i].Tstarts[j]) && (psl->Tends[k]==psls[i].Tends[j]))
				{
					int ret = check_mismatch(i,psl) ;
					if ((ret<=0) && (psl->to_be_removed==psls[i].to_be_removed))
						match=1 ;
				} ;
			} ;
		} ;
		if (match)
		{
			found[num_match]=i ;
			num_match++ ;
			assert(num_match<FOUND_BUF_SIZE) ;
		} ;
    }
	return num_match ;
} ;


/* merges a list of psls, puts the exon in right order and performs a few checks */
int merge_contigous(int idx, int num, int *found, struct psl_merged *mpsl) 
{
	mpsl->Qname = strdup(psls[idx].Qname) ;
	mpsl->contig_num = psls[idx].contig_num ;
	mpsl->strand = psls[idx].strand ;
	int Tstarts[FOUND_BUF_SIZE], Tends[FOUND_BUF_SIZE] ;
	int num_confirmed[FOUND_BUF_SIZE] ;
	int i, blockcnt=0 ;
	assert(num>0) ;
	
	mpsl->to_be_removed = psls[found[0]].to_be_removed ;
	
	for (i=0; i<num; i++) 
    {
		int j,k ;
		
		assert(psls[found[i]].to_be_removed==mpsl->to_be_removed) ;
		
		psls[found[i]].covered ++ ;
		for (j=0; j<psls[found[i]].blockcnt; j++)
		{
			int already_there = 0 ;
			
			for (k=0; k<blockcnt; k++)
				if ((psls[found[i]].Tstarts[j]==Tstarts[k]) || 
					(psls[found[i]].Tends[j]==Tends[k]))
				{
					assert((psls[found[i]].Tstarts[j]==Tstarts[k]) && 
						   (psls[found[i]].Tends[j]==Tends[k])) ;
					already_there = 1 ;
					break ;
				} ;
			if (already_there) // insert at right position
			{
				num_confirmed[k]++ ;
				fprintf(stdout, "already there %i\n",k) ;
			}
			else
			{
				int insert_idx = blockcnt ;
				int e_overlap = -1 ;
				int s_overlap = -1 ;
				int s_inclusion = -1 ;
				int e_inclusion = -1 ;
				for (k=0; k<blockcnt; k++) 
				{
					if (psls[found[i]].Tstarts[j]<Tstarts[k])
					{
						if (psls[found[i]].Tends[j]>Tends[k])
						{
							fprintf(stdout, "s_inclusion: \n") ;
							fprintf(stdout, "%i-%i [%s,%i,%i]\n", 
									psls[found[i]].Tstarts[j], psls[found[i]].Tends[j],
									psls[found[i]].Qname, j,psls[found[i]].blockcnt) ;
							fprintf(stdout, "%i-%i [%i,%i]\n", 
									Tstarts[k], Tends[k], 
									k, blockcnt) ;
							s_inclusion = k ;
							//Tstarts[k] = psls[found[i]].Tstarts[j] ;
							//Tends[k] = psls[found[i]].Tends[j] ;
							break ;
						} 
						if ((psls[found[i]].Tends[j]<Tends[k]) &&
							(psls[found[i]].Tends[j]>Tstarts[k]))
						{
							fprintf(stdout, "s_overlap: \n") ;
							fprintf(stdout, "%i-%i [%s,%i,%i]\n", 
									psls[found[i]].Tstarts[j], psls[found[i]].Tends[j],
									psls[found[i]].Qname, j,psls[found[i]].blockcnt) ;
							fprintf(stdout, "%i-%i [%i,%i]\n", 
									Tstarts[k], Tends[k], 
									k, blockcnt) ;
							s_overlap = k ;
							//Tstarts[k] = psls[found[i]].Tstarts[j] ;
							break ;
						} 
						insert_idx = k ;
						break ;
					} ;
					if (psls[found[i]].Tends[j]>Tends[k])
					{
						if (psls[found[i]].Tstarts[j]<Tstarts[k])
						{
							fprintf(stdout, "e_inclusion: \n") ;
							fprintf(stdout, "%i-%i [%s,%i,%i]\n", 
									psls[found[i]].Tstarts[j], psls[found[i]].Tends[j],
									psls[found[i]].Qname, j,psls[found[i]].blockcnt) ;
							fprintf(stdout, "%i-%i [%i,%i]\n", 
									Tstarts[k], Tends[k], 
									k, blockcnt) ;
							//e_inclusion = k ;
							break ;
						} 
						if ((psls[found[i]].Tstarts[j]<Tends[k]) &&
							(psls[found[i]].Tstarts[j]>Tstarts[k]))
						{
							fprintf(stdout, "e_overlap: \n") ;
							fprintf(stdout, "%i-%i [%s,%i,%i]\n", 
									psls[found[i]].Tstarts[j], psls[found[i]].Tends[j],
									psls[found[i]].Qname, j,psls[found[i]].blockcnt) ;
							fprintf(stdout, "%i-%i [%i,%i]\n", 
									Tstarts[k], Tends[k], 
									k, blockcnt) ;
							e_overlap = k ;
							//Tends[k] = psls[found[i]].Tends[j] ;
							break ;
						} 
					} ;
				} ;
				assert((s_overlap==-1) && (e_overlap==-1) &&
					   (s_inclusion=-1) && (e_inclusion==-1)) ;
				
				if (insert_idx == blockcnt)
				{
					Tstarts[blockcnt]=psls[found[i]].Tstarts[j] ;
					Tends[blockcnt]=psls[found[i]].Tends[j] ;
					num_confirmed[blockcnt] = 1 ;
					fprintf(stdout, "append %i - %i \n", psls[found[i]].Tstarts[j], psls[found[i]].Tends[j]) ;
					blockcnt ++ ;
				}
				else
				{
					// move and insert 
					copy_int(&Tstarts[insert_idx+1], &Tstarts[insert_idx], (blockcnt-insert_idx)) ;
					copy_int(&Tends[insert_idx+1], &Tends[insert_idx], (blockcnt-insert_idx)) ;
					copy_int(&num_confirmed[insert_idx+1], &num_confirmed[insert_idx], (blockcnt-insert_idx)) ;
					Tstarts[insert_idx] = psls[found[i]].Tstarts[j] ;
					Tends[insert_idx]   = psls[found[i]].Tends[j] ;
					fprintf(stdout, "insert at %i: %i - %i \n", insert_idx, psls[found[i]].Tstarts[j], psls[found[i]].Tends[j]) ;
					num_confirmed[insert_idx] = 1 ;
					blockcnt++ ;
				} ;
			} ;
		} ;
    } ;
	for (i=0; i<blockcnt; i++)
	{
		fprintf(stdout, "%i - %i\n", Tstarts[i], Tends[i]);
	} ;
	/* check for overlaps in resulting sequence */
	for (i=0; i<blockcnt-1; i++)
    {
		assert(Tends[i]<Tstarts[i+1]) ;
		if (mpsl->strand=='+')
			assert(Tstarts[i]<=Tends[i]) ;
		else
			assert(Tstarts[i]>=Tends[i]) ;
    } ;
	
	mpsl->blockcnt = blockcnt ;
	mpsl->Tstarts = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->Tstarts, Tstarts, blockcnt) ;
	mpsl->Tends = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->Tends, Tends, blockcnt) ;
	mpsl->num_confirmed = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->num_confirmed, num_confirmed, blockcnt) ;
	mpsl->valid = 1 ;
	
	return blockcnt ;
} 

void merge_continous_mpsls()
{
	int i, j ;
	for (i=0; i<mpsl_num; i++)
		for (j=0; j<mpsl_num; j++)
		{
			if ((mpsls[i].Tends[mpsls[i].blockcnt-1]>mpsls[j].Tstarts[0]) &&
				(mpsls[i].Tstarts[mpsls[i].blockcnt-1]<mpsls[j].Tstarts[0]) &&
				(mpsls[i].blockcnt>=2) && (mpsls[j].blockcnt>=2))
				//(mpsls[i].valid) && (mpsls[j].valid))
			{
				struct psl_merged mpsl ;
				int blockcnt = mpsls[i].blockcnt-1+mpsls[j].blockcnt ;
				
				mpsl.blockcnt = blockcnt ;
				char buf[200] ;
				sprintf(buf, "%s-%s", mpsls[i].Qname, mpsls[j].Qname) ;
				mpsl.Qname = strdup(buf) ;
				mpsl.contig_num = mpsls[i].contig_num ;
				mpsl.strand = mpsls[i].strand ;
				
				fprintf(stdout, "merging mpsls %s [%i] and %s [%i] -> %s [%i]\n",
						mpsls[i].Qname, mpsls[i].blockcnt, 
						mpsls[j].Qname, mpsls[j].blockcnt, 
						mpsl.Qname, blockcnt) ;
				
				mpsl.Tstarts = (int*)malloc(sizeof(int)*blockcnt) ;
				copy_int(mpsl.Tstarts, mpsls[i].Tstarts, mpsls[i].blockcnt) ;
				copy_int(mpsl.Tstarts+mpsls[i].blockcnt, mpsls[j].Tstarts+1, (mpsls[j].blockcnt-1)) ;
				
				mpsl.Tends = (int*)malloc(sizeof(int)*blockcnt) ;
				copy_int(mpsl.Tends, mpsls[i].Tends, (mpsls[i].blockcnt-1)) ;
				copy_int(mpsl.Tends+mpsls[i].blockcnt-1, mpsls[j].Tends, mpsls[j].blockcnt) ;
				
				mpsl.num_confirmed = (int*)malloc(sizeof(int)*blockcnt) ;
				copy_int(mpsl.num_confirmed, mpsls[i].num_confirmed, mpsls[i].blockcnt) ;
				copy_int(mpsl.num_confirmed+mpsls[i].blockcnt, mpsls[j].num_confirmed+1, (mpsls[j].blockcnt-1)) ;
				mpsl.valid = 1 ;
				mpsl.to_be_removed = 0 ;
				mpsls[mpsl_num]=mpsl ;
				mpsl_num++ ;
				mpsls[i].valid = 0 ;
				mpsls[j].valid = 0 ;
			} ;
		} ;
	
} ;

int check_mpsl(struct psl_merged * mpsl)
{
	if (!mpsl->valid)
		return 0 ;
	return 1 ;
}

void print_mpsl(struct psl_merged * mpsl, FILE* fd, int psl_idx)
{
	if (!mpsl->valid)
		return ;
	if (!check_mpsl(mpsl))
		return ;
	
	int i ;
	/* output all exons */
	for (i=0; i<mpsl->blockcnt; i++) 
	{
		if (mpsl->strand=='+')
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t0\t%c\t%i\tGR-%s-%i%c.%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], 
					mpsl->Tstarts[i], mpsl->Tends[i], mpsl->strand, mpsl->num_confirmed[i],
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, 
					i, mpsl->blockcnt, mpsl->to_be_removed) ;
			if (mpsl->to_be_removed)
				fprintf(stdout, "exon %i of %s is marked as to be removed\n", i, mpsl->Qname) ;
		}
		else	
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t0\t%c\t%i\tGR-%s-%i%c.%i\t%i\t%i\t%i\n", contig_names[mpsl->contig_num], 
					mpsl->Tends[i], mpsl->Tstarts[i], mpsl->strand, mpsl->num_confirmed[i],
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, 
					i, mpsl->blockcnt, mpsl->to_be_removed) ;
			if (mpsl->to_be_removed)
				fprintf(stdout, "exon %i of %s is marked as to be removed\n", i, mpsl->Qname) ;
		}
	}
	
	/* output all introns */
	for (i=0; i<mpsl->blockcnt-1; i++) 
	{
		int start_diff=0 ;
		int end_diff = 0 ;
		int ok = 0 ;
		
		if (mpsl->strand=='+')
			ok = check_intron(mpsl->contig_num, mpsl->Tends[i]+1, 
							  mpsl->Tstarts[i+1]-1, mpsl->strand,
							  &start_diff, &end_diff) ;
		else
			ok = check_intron(mpsl->contig_num, mpsl->Tstarts[i]+1,
							  mpsl->Tends[i+1]-1, mpsl->strand,
							  &start_diff, &end_diff) ;

		if (ok!=1)
		{
			fprintf(stderr,"intron not ok: %i [%i,%i]\n", ok, start_diff, end_diff) ;
			//exit(-1) ;
		} ;
		
		int intron_type ;
		
		if (mpsl->strand=='+')
			intron_type = get_intron_type(mpsl->contig_num, mpsl->Tends[i]+1, 
							  mpsl->Tstarts[i+1]-1, mpsl->strand) ;
		else
			intron_type = get_intron_type(mpsl->contig_num, mpsl->Tstarts[i]+1,
							  mpsl->Tends[i+1]-1, mpsl->strand) ;

		if (intron_type==1)
			num_intron_type1++ ;
		if (intron_type==2)
			num_intron_type2++ ;
		
		if (mpsl->strand=='+')
		{
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t0\t%c\t%i\tGR-%s-%i%c.%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], mpsl->Qname, 
					mpsl->Tends[i]+1, mpsl->Tstarts[i+1]-1, mpsl->strand, 
					min(mpsl->num_confirmed[i], mpsl->num_confirmed[i+1]),
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, 
					i, mpsl->blockcnt-1, mpsl->to_be_removed) ;
			if (mpsl->to_be_removed)
				fprintf(stdout, "intron %i of %s is marked as to be removed\n", i, mpsl->Qname) ;
		}
		else
		{			
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t0\t%c\t%i\tGR-%s-%i%c.%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], mpsl->Qname, 
					mpsl->Tstarts[i]+1, mpsl->Tends[i+1]-1, mpsl->strand, 
					min(mpsl->num_confirmed[i], mpsl->num_confirmed[i+1]),
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, 
					i, mpsl->blockcnt-1, mpsl->to_be_removed) ;
			if (mpsl->to_be_removed)
				fprintf(stdout, "intron %i of %s is marked as to be removed\n", i, mpsl->Qname) ;
		}
	} ;
	
}

int find_contigous(int idx) 
{
	int found[FOUND_BUF_SIZE] ;
	
	// find overlaps
	int num = 1;//find_match(&psls[idx], found) ;
	found[0]=idx ;
	
	fprintf(stdout, "%i. num_match = %i -> ", idx, num) ;
	assert(num>0) ;
	{
		int i;
		for (i=0; i<num; i++)
			fprintf(stdout, "%s ", psls[found[i]].Qname) ;
		fprintf(stdout, "\n") ;
	}
	
	int changed = 1 ;
	while (changed)
    {
		changed = 0 ;
		
		int i, num2 = num ;
		int found2[FOUND_BUF_SIZE] ;
		copy_int(found2, found, num) ;
		
		num = 1 ;
		found[0] = idx ;
		
		/* remove mismatching items */
		int j ;
		for (j=0; j<num2; j++)
		{
			int include = 1 ;
			int k;
			for (k=0; k<num; k++) 
			{
				if (check_mismatch(found2[j], &psls[found[k]])>0)
					include = 0 ;
				if (found2[j]==found[k])
					include = -1 ;
			} ;
			if (include==1)
			{
				found[num]=found2[j] ;
				num++ ;
			} 
			else 
				if (include==0)
					changed=1 ;
		} ;
		fprintf(stdout, "%i -> ", num) ;
		
		/* add new items that match to new items */
		// for (i=0; i<num; i++)
		if (0) // slow and does not help
		{
			num2 = find_match(&psls[found[i]], found2) ;
			int j ;
			for (j=0; j<num2; j++)
			{
				int include = 1 ;
				int k;
				for (k=0; k<num; k++) 
				{
					if (check_mismatch(found2[j], &psls[found[k]])>0)
						include = 0 ;
					if (found2[j]==found[k])
						include = 0 ;
				} ;
				if (include)
				{
					found[num]=found2[j] ;
					num++ ;
					changed=1 ;
				} ;
			} ;
		} ;
    } ;

	merge_contigous(idx, num, found, &mpsls[mpsl_num]) ;
	fprintf(stdout, "%i [%i]\n", num, mpsls[mpsl_num].blockcnt) ;
	mpsl_num++ ;
	
	return num ;
} ;


int main(int argc, char**argv)
{
	FILE * psl_file=NULL, *out_file=NULL ;
	int i ;
	char * genome_config_fname=NULL ;

	if (argc<4)
    {
		fprintf(stderr, "usage: %s <psl-file> <contig_name> <strand> [<output_fname> [<genome_config_fname]] \n", argv[0]) ;
		exit(-1) ;
    } ;

	// genome config
	if (argc<6)
		genome_config_fname = "genome.config" ;
	else
		genome_config_fname = argv[5] ;
	if (init_genome(genome_config_fname)!=0)
	{
		fprintf(stderr, "genome initialization failed (%s)\n",genome_config_fname) ;
		exit(-1) ;
	}

	// output file 
	if (argc<5)
		out_file = stdout ;
	else
		if (strcmp(argv[4],"STDOUT")==0)
			out_file = stdout ;
		else
			out_file = fopen(argv[4], "w+") ;
	if (out_file==NULL)
    {
		fprintf(stderr, "could not open %s\n", argv[4]) ;
		exit(-1) ;
    } ;

	// psl file
	psl_file=fopen(argv[1], "r") ;
	if (!psl_file)
    {
		fprintf(stderr,"could not open %s\n", argv[1]) ;
		exit(-1) ;
    } ;
	
	int contig_num = get_contig_num(argv[2]) ;
	assert(strlen(argv[3])==1) ;
	char strand = argv[3][0] ;
	
	assert((strand=='+') || (strand=='-')) ;
	assert((contig_num>=0) && (contig_num<num_contigs)) ;
	
	fprintf(stdout, "Considering %s, strand %c\n", contig_names[contig_num], strand) ;
	
	contig_files[contig_num]=fopen(contig_fnames[contig_num],"r") ;
	if (!contig_files[contig_num])
	{
		fprintf(stderr,"could not open %s\n", contig_fnames[contig_num]) ;
		exit(-1) ;
	} ;
		
	read_psl(psl_file, contig_num, strand) ;
	
	for (i=0; i<psl_num; i++)
	{
		if (psls[i].covered==0)
		{
			//fprintf(stderr, "%d\n", i) ;
			find_contigous(i) ;
		}
	}

//	merge_continous_mpsls() ;
	
	for (i=0; i<mpsl_num; i++)
		if (mpsls[i].valid)
			print_mpsl(&mpsls[i], out_file, i) ;

	
	free_all_psl() ;
	free_all_mpsl() ; 
	
	fclose(psl_file) ;
	if (argc>=6)
		fclose(out_file) ;
	//if (argc>=6)
	//fclose(microexons_file) ;
	
	fprintf(stderr, "num_intron_type1=%i\n", num_intron_type1) ;
	fprintf(stderr, "num_intron_type2=%i\n", num_intron_type2) ;
		
	cleanup_genome() ;

	return 0;
} 
