function count_nmd(info,genesfilename)
% function count_nmd(info,genesfilename)
%
% Generates all possible isoforms from the splicegraph,
% and locate all the start and stop codons.
% The reason for genesfilename is so that we can choose between the
% EST confirmed genes and the predicted genes.
%
% Adds the following fields to the genes structure:
%  - isoforms - a list of exons for each isoform (path through graph)
% For each isoform:
%  - lastEEpos - corresponding location of the last exon-exon junction
%  - isoORFlength - length of the longest open reading frame
%  - isostartpos - start position of the ORF
%  - isostoppos - stop position of the ORF
% For each exon:
%  - tstop - isostoppos-lastEEpos for each isoform.
%
% Written by: Cheng Soon Ong, 4 October 2006
% Modified on: 12 November 2006, 10 May 2007

addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/projects/splicing/splicegraphs

MAX_EXONS = 100;

genome_info = init_genome(info);
fprintf('Loading %s/%s\n',genome_info.basedir,genesfilename);
load(sprintf('%s/%s',genome_info.basedir,genesfilename),'genes');



fprintf('Processing...\n')
geneORFlength = zeros(length(genes),1);
for ix = 1:length(genes)
  if mod(ix,1000)==0
    fprintf('.');
  end
  %fprintf('%d\n',ix);
  
  % get all the isoforms
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  if size(vertices,2) > MAX_EXONS
    fprintf('skipping gene %d, with %d exons\n',ix,size(vertices,2));
    continue;
  end
  strand = genes(ix).strands(1);
  isoforms = find_all_isoforms(vertices,edges,strand);
  

  lastEEpos = zeros(1,length(isoforms));
  isostartpos = zeros(1,length(isoforms));
  isostoppos = zeros(1,length(isoforms));
  isoORFlength = zeros(1,length(isoforms));
  for ixf = 1:length(isoforms)
    str = load_genomic(genes(ix).chr,strand,vertices(1,isoforms{ixf})',vertices(2,isoforms{ixf})');

    % get location of last exon-exon junction
    if strand == '+'
      lastEEpos(ixf) = length(str) - (vertices(2,isoforms{ixf}(end))-vertices(1,isoforms{ixf}(end)));
    else
      lastEEpos(ixf) = length(str) - (vertices(2,isoforms{ixf}(1))-vertices(1,isoforms{ixf}(1)));
    end
    % find the longest open reading frame
    [isostartpos(ixf),isostoppos(ixf),isoORFlength(ixf)] = get_longest_orf(str);
  end
  genes(ix).isoforms = isoforms;
  genes(ix).lastEEpos = lastEEpos;
  genes(ix).isoORFlength = isoORFlength;
  genes(ix).isostartpos = isostartpos;
  genes(ix).isostoppos = isostoppos;


  % find all the transcription stops
  if strand == '-'
    tstop = [];
    for ixv = 1:size(vertices,2)
      if sum(edges(ixv,1:ixv-1)) == 0
	tstop(end+1) = ixv;
      end
    end
  else
    tstop = [];
    for ixv = 1:size(vertices,2)
      if sum(edges(ixv+1:end,ixv)) == 0
	tstop(end+1) = ixv;
      end
    end
  end


  % report stop codon distribution for each transcription end.
  for ixe = tstop
    stoplist = [];
    for ixf = 1:length(isoforms)
      if ((strand == '+') && (isoforms{ixf}(end) == ixe)) ...
	    || ((strand == '-') && (isoforms{ixf}(1) == ixe))
	stoplist(end+1) = isostoppos(ixf)-lastEEpos(ixf);
      end
    end
    genes(ix).tstop{ixe} = stoplist;
  end
  

  % some genome level statistics
  %[geneORFlength(ix),ORFidx] = max(isoORFlength);
end

all_tstop = [genes(:).tstop];
array_tstop = [all_tstop{:}];
fprintf('\nSaving %s/genes_ORF.mat  ',genome_info.basedir);
save(sprintf('%s/genes_ORF.mat',genome_info.basedir),'genes','array_tstop');
fprintf('done.\n');





