function all_genes = process_genes(PAR,genes,old_signals)
  
% all_genes = process_genes(PAR,genes)
%  
% merges gene structures from different sources (annotation and ESTs), performs merging by
% name and collocalization.
% returns genes in genome coordinates with splicegraphs
%  
% INPUT PAR.FN.input.fn_genome_config
%       PAR.FN.input.fn_genes_anno  
%       PAR.FN.input.fn_genes_est
%       PAR.label_source.use_anno 
%       PAR.label_source.use_confirmed 
%       PAR.label_source.tss.ests
%       PAR.label_source.cleave.ests
%       PAR.label_source.tss.fulllength
%       PAR.label_source.cleave.fulllength
%       PAR.label_source.tss.db
%       PAR.label_source.cleave.db
%
% functions called: init_genome
%                   init_genes
%                   map_two_genes_structures
%                   cluster_CG_matches
%                   parse_fulllength_cDNA
%                   parse_db
%                   check_genes
%                   build_valid_splicegraph  
%                   infer_splice_graph 
%                   alt_const 
  
genome_info =  init_genome(PAR.FN.input.fn_genome_config) ;
all_genes = init_genes(1,PAR);
which_checks = [];
which_checks.transacc = 0;
build_graph = 1;
if nargin>1
  all_genes  = map_two_genes_structures(all_genes, genes,genome_info) ;
  if isfield(genes,'splicegraph') 
    build_graph=0;
  end
else
  old_signals = [];
  %%% LOAD ANNOTATION  
  if ~fexist(PAR.FN.output.fn_genes_anno) & PAR.label_source.use_anno 
    error('annotation not found\n')
  elseif ~PAR.label_source.use_anno 
    fprintf('\t no annotation used\n')
  else
    fprintf('\t loading annotation\n')
    genes = load_struct(PAR.FN.output.fn_genes_anno,'genes');
  
    all_genes  = map_two_genes_structures(all_genes, genes,genome_info) ;
  end
  clear genes
  
  % keyboard
  %%% LOAD TRANSCRIPT GENES (confirmed genes)
  if ~fexist(PAR.FN.output.fn_genes_conf) & PAR.label_source.use_confirmed 
    error('confirmed_genes not found')
  elseif ~PAR.label_source.use_confirmed 
    warning('no (EST-) confirmed genes used!')
    
    if 0 % isfield(all_genes,'splicegraph') % annotated genes can be merged
      fprintf(1,'splicegraph from annotated genes used;\n')
      fprintf(1,'no new splicegraph built;\n')
      build_graph = 0;
    end
  elseif PAR.label_source.use_confirmed 
    fprintf('\t loading confirmed genes\n')
    C.genes = load_struct(PAR.FN.output.fn_genes_conf,'genes');
    
    %%% MAP TRANSCRIPT GENES ON ANNOTATION
    all_genes  = map_two_genes_structures(all_genes, C.genes,genome_info) ;
    for i=1:length(all_genes)
      all_genes(i).id = i;
    end
    %% add transformed paralogs
    if isfield(C.genes,'paralogs')
      idx1 = find(~cellfun('isempty',{C.genes.paralogs}));
      idx2 = find(~cellfun('isempty',{all_genes.confgenes_id}));
      idx3 = [all_genes(idx2).confgenes_id];
      if length(idx3)~=length(idx2)
        assert(length(idx3)>length(idx2))
        ll = arrayfun(@(x) (length(x.confgenes_id)), all_genes(idx2), 'UniformOutput', false);
        ll = [ll{:}];
        ss  = min(find(ll>1));
        idx2_new = idx2(1:ss-1);
        for j=ss:length(idx2)
          idx2_new = [idx2_new,repmat(idx2(j),1,ll(j))];
        end
        assert(length(idx3)==length(idx2_new))
        idx2 = idx2_new; 
        clear idx2_new
      end
      [temp,ii1,ii2] = intersect(idx1,idx3);
      assert(length(ii1)==length(idx1))
      
      for j=1:length(ii1)
        paralogs =  C.genes(idx1(ii1(j))).paralogs;
        [temp,temp,jj3] = intersect(paralogs,idx3);
        assert(length(temp)==length(paralogs))
      all_genes(idx2(ii2(j))).paralogs = idx3(jj3);
      end
    end
    
    which_checks.intron_length = 1;
    which_checks.splicesites = 1;
    which_checks.gene_length = 1;
  end
  if PAR.label_source.use_confirmed  & ~PAR.label_source.use_anno & ...
        isfield(C.genes,'splicegraph')
    fprintf(1,'splicegraph from confirmed genes used;\n')
    fprintf(1,'no new splicegraph built;\n')
    build_graph = 0;
  end
end

fprintf('getting additional label information\n')
all_genes = addlabels2genes(all_genes, PAR,old_signals);

if PAR.add_signals.tis |PAR.add_signals.cdsStop
  % which_checks.orf=1;
end

if PAR.add_signals.transacc
  all_genes = addPredTransacc2genes(all_genes,PAR);
  all_genes = mergePredAnnoTransacc(all_genes,PAR);
  which_checks.transacc=1;
end
% if % 0%ismember(PAR.Signal_name,{'tis','Stop'})
%   all_genes1 = add_cand_cds( PAR, all_genes );
%    which_checks.orf = 1;
% end

% mark genes with invalid introns and other properties

% make splicegraph
if build_graph
  fprintf(1,'building splicegraph') 
  for i=1:length(all_genes)
    all_genes(i).is_valid = 1;
    for j=1:length(all_genes(i).transcripts)
      all_genes(i).transcript_valid(j) = 1;
    end
  end
  all_genes = build_valid_splicegraph(all_genes) ;
  all_genes = infer_splice_graph_caller(all_genes);
  for ix=1:length(all_genes)
    if isempty(all_genes(ix).splicegraph{1})
      continue
    end
    [dummy,exon_order] = sort(all_genes(ix).splicegraph{1}(1,:),2,'ascend');
    all_genes(ix).splicegraph{1} = all_genes(ix).splicegraph{1}(:,exon_order);
    all_genes(ix).splicegraph{2} = all_genes(ix).splicegraph{2}(exon_order,exon_order);
  end
  all_genes = alt_const(all_genes) ;
  for j=1:length(all_genes)
    all_genes(j).id=j;
  end
  
  % mark genes with invalid graph
  which_checks.graph = 1;
end
% keyboard
all_genes = check_genes(all_genes, genome_info,which_checks) ;

