/* takes a fasta file and outputs a new fasta file on stdout without duplicates 
   * a crc64 hash is used 
   * reads the file twice
   * needs much less memory than the previous matlab implementation
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#include "common.h"

/*
 * Improved calculation of CRC-64 values for protein sequences
 * By David T. Jones (dtj@cs.ucl.ac.uk)  - September 28th 2002
 * 
 * Modified from code at URL:
 * ftp://ftp.ebi.ac.uk/pub/software/swissprot/Swissknife/old/SPcrc.tar.gz
 */

/* If you want to try the old CRC-64 function, currently employed in
   SWISSPROT/TrEMBL then uncomment the next line */
/* #define OLDCRC */

#ifdef OLDCRC
  #define POLY64REV0xd800000000000000ULL
  #define INITIALCRC0x0000000000000000ULL
#else
  #define POLY64REV     0x95AC9329AC4BC9B5ULL
  #define INITIALCRC    0xFFFFFFFFFFFFFFFFULL
#endif



unsigned long long crc64(char *seq)
{
	
    int i, j;
	
    unsigned long long crc = INITIALCRC, part;
	
    static int init = 0;
	
    static unsigned long long CRCTable[256];
	
    
    if (!init)
    {
		init = 1;
		for (i = 0; i < 256; i++)
		{
			part = i;
			for (j = 0; j < 8; j++)
			{
				if (part & 1)
					part = (part >> 1) ^ POLY64REV;
				else
					part >>= 1;
			}
			CRCTable[i] = part;
		}
    }
    
    while (*seq)
		crc = CRCTable[(crc ^ *seq++) & 0xff] ^ (crc >> 8);

    return crc ;
}

void write_fasta(FILE* fa, char* seq)
{
	int len=strlen(seq) ;
	int i = 0 ;
	for (i=0; i<len; i++)
	{
		fputc(seq[i], fa) ;
		if (i%80==79)
			fprintf(fa,"\n") ;
	} ;
	if ((i-1)%80!=79)
		fprintf(fa,"\n") ;
}

void myqsort(unsigned long long* output, int size)
{
	if (size==2)
	{
		if (output[0] > output [1])
		{
			unsigned long long c=output[0] ;
			output[0]=output[1] ;
			output[1]=c ;
		}
	}
	else
	{
		//unsigned long long split=output[(int) ((size*rand())/(RAND_MAX+1.0))];
		unsigned long long split=output[size/2];
		
		int left=0;
		int right=size-1;
		
		while (left<=right)
		{
			while (output[left] < split)
				left++;
			while (output[right] > split)
				right--;
			
			if (left<=right)
			{
				unsigned long long c=output[left] ;
				output[left]=output[right] ;
				output[right]=c ;
				left++;
				right--;
			}
		}
		
		if (right+1> 1)
			myqsort(output,right+1);
		
		if (size-left> 1)
			myqsort(&output[left],size-left);
	}
}

int binary_search(unsigned long long* output, int size, unsigned long long elem)
{
	int start=0;
	int end=size-1;
	
	if (size<1)
		return -1;
	
	while (start<end)
	{
		int middle=start+(end-start)/2; 
		
		if (output[middle]>elem)
			end=middle-1;
		else if (output[middle]<elem)
			start=middle+1;
		else
			return middle;
	}
	
	if (start<size && output[start]==elem)
		return start;
	else
		return -1;
}

int main(int argc, char**argv)
{
	const int max_array_size=30000000;
	const int max_seq_len = 1000000 ;
	
	if (argc!=2)
	{
		fprintf(stderr, "usage: %s <fasta-filename>\n", argv[0]) ;
		exit(-1) ;
	}

	int num_elem=0 ;
	unsigned long long *crc64_array = (unsigned long long *)malloc(max_array_size*sizeof(unsigned long long)) ;
	char *taken_array = (char *)calloc(max_array_size,sizeof(char)) ;

	FILE *fd=fopen(argv[1], "r") ;
	char *seq = (char*)calloc(max_seq_len,sizeof(char)) ;
	int seq_len=0 ;
	char last_header[1000]="" ;
	
	while (!feof(fd))
	{
		const size_t max_len=1000 ;
		char line[max_len] ;
		read_line(fd, line, max_len) ;
		if (line[0]!='>')
		{
			char *ptr=line ;
			while (*ptr)
			{
				char c=*ptr ;
				switch (c)
				{
				case 'A': 
				case 'a': c='a' ; break ;
				case 'C': 
				case 'c': c='c' ; break ;
				case 'G': 
				case 'g': c='g' ; break ;
				case 'T': 
				case 't': c='t' ; break ;
				case 'N': 
				case 'n': c='n' ; break ;
				case 'M': 
				case 'm': c='m' ; break ;
				case 'R': 
				case 'r': c='r' ; break ;
				case 'W': 
				case 'w': c='w' ; break ;
				case 'S': 
				case 's': c='s' ; break ;
				case 'Y': 
				case 'y': c='y' ; break ;
				case 'K': 
				case 'k': c='k' ; break ;
				case 'V': 
				case 'v': c='v' ; break ;
				case 'H': 
				case 'h': c='h' ; break ;
				case 'D': 
				case 'd': c='d' ; break ;
				case 'B': 
				case 'b': c='b' ; break ;
				case 'X': 
				case 'x': c='x' ; break ;
				default:
					fprintf(stderr,"wrong letter %c '%s'\n", c, line) ;
					exit(-1)  ;
				}
				seq[seq_len++]=c ;
				assert(seq_len+1<max_seq_len) ;
				seq[seq_len]=0 ;
				ptr++ ;
			}
		}

		if (line[0]=='>' || feof(fd))
		{
			if (strlen(seq)!=0)
			{
				//if (strlen(seq)>5000)
				//	fprintf(stderr, "%s -> %i\n", last_header, (int)strlen(seq)) ;
				//fprintf(stdout, "'%s':%s\n", last_header, seq) ;
				crc64_array[num_elem++]=crc64(seq) ;
			}
			seq_len=0 ;
			seq[0]=0 ;
			strcpy(last_header, line) ;
		}
	}
	fprintf(stderr, "number of sequences: %i\n", num_elem) ;
	
	fprintf(stderr, "sorting ...") ;
	myqsort(crc64_array, num_elem) ;
	fprintf(stderr, "removing duplicates ...") ;
	int i ;
	for (i=1; i<num_elem; i++)
		if (crc64_array[i-1]==crc64_array[i])
			crc64_array[i-1] = INITIALCRC ;
	fprintf(stderr, "sorting again ...") ;
	myqsort(crc64_array, num_elem) ;

	for (i=0; i<num_elem; i++)
		if (crc64_array[i]==INITIALCRC)
		{
			num_elem=i+1 ;
			break ;
		}
	
	fprintf(stderr, "\nnumber of unique crc64's: %i\n", num_elem) ;

	fprintf(stderr, "reading file from beginning ...") ;
	int unique_seqs = 0 ;
	fseek(fd, 0L, SEEK_SET) ;
	while (!feof(fd))
	{
		const size_t max_len=1000 ;
		char line[max_len] ;
		read_line(fd, line, max_len) ;
		if (line[0]!='>')
		{
			char *ptr=line ;
			while (*ptr)
			{
				char c=*ptr ;
				switch (c)
				{
				case 'A': 
				case 'a': c='a' ; break ;
				case 'C': 
				case 'c': c='c' ; break ;
				case 'G': 
				case 'g': c='g' ; break ;
				case 'T': 
				case 't': c='t' ; break ;
				case 'N': 
				case 'n': c='n' ; break ;
				case 'M': 
				case 'm': c='m' ; break ;
				case 'R': 
				case 'r': c='r' ; break ;
				case 'W': 
				case 'w': c='w' ; break ;
				case 'S': 
				case 's': c='s' ; break ;
				case 'Y': 
				case 'y': c='y' ; break ;
				case 'K': 
				case 'k': c='k' ; break ;
				case 'V': 
				case 'v': c='v' ; break ;
				case 'H': 
				case 'h': c='h' ; break ;
				case 'D': 
				case 'd': c='d' ; break ;
				case 'B': 
				case 'b': c='b' ; break ;
				case 'X': 
				case 'x': c='x' ; break ;
				default:
					fprintf(stderr,"wrong letter %c '%s'\n", c, ptr) ;
					exit(-1)  ;
				}
				seq[seq_len++]=c ;
				assert(seq_len+1<max_seq_len) ;
				seq[seq_len]=0 ;
				ptr++ ;
			}
		}

		if (line[0]=='>' || feof(fd))
		{
			if (strlen(seq)!=0)
			{
				unsigned long long crc = crc64(seq) ;
				int idx = -1 ;
				idx = binary_search(crc64_array, num_elem, crc) ;
				assert(idx>=0) ;
				if (taken_array[idx]==0)
				{
					unique_seqs++ ;
					taken_array[idx]=1 ;
					fprintf(stdout, "%s\n", last_header) ;
					write_fasta(stdout, seq) ;
				}
			}
			seq_len=0 ;
			seq[0]=0 ;
			strcpy(last_header, line) ;
		}
	}
	fprintf(stderr, "Done\n%i unique sequences\n", unique_seqs) ;
	return 0 ;
}
