#!/usr/bin/env python

"""A gene structure which is a copy of the matlab version.

Written by: Cheng Soon Ong
"""

#import pdb
import sys
import scipy.io
import numpy
from numpy import triu, tril

from io_pickle import load, save
from genome_utils import load_genomic
from graph import dfs_visit

class SpliceGraph:
    """Holds information describing the (splicegraph) ordered directed acyclic graph.

    Members:
    exons - a (2 by n) matrix representing the positions of the exon boundaries.
    introns - an adjacency matrix (symmetric) denoting the introns.
    """
    def __init__(self,inarrays):
        self.exons = inarrays[0]
        self.introns = inarrays[1]
        
    def __repr__(self):
        return 'exons:\n' + repr(self.exons) + '\nintrons:\n' + repr(self.introns) + '\n'

class Gene:
    """Holds information for a particular region of the genome.

    Members:
    name - an identifier based on the transcript names
    chr - the chromosome name
    start - left most position
    stop - right most position
    transcripts - a list of identifiers of ESTs
    exons - a list of (n by 2) matrices where n is the number of exons.
            The entries of this matrix are the positions of the splice sites.
    exons_confirmed - a list of the same size as exons denoting the number
            confirmations of the corresponding splice site.
    strands - the strand of each transcript.
    terminal_info - ??? (2 by n) matrix, where n is the number of transcripts.
    is_cDNA - a boolean array denoting whether a transcript comes from cDNA or not.
    has_cDNA - a boolean denoting whether any transcript comes from cDNA
    transcript_valid - a boolean array denoting whether a transcript is valid
    is_valid - a boolean denoting whether the gene is valid.
    splicegraph.exons - a (2 by n) matrix showing the position of the exons
    splicegraph.introns - a (n by n) adjacency matrix
    is_alt_spliced - a boolean denoting whether gene is alternatively spliced
    is_alt - a boolean denoting whether gene has alternative isoforms
    has_paralog - a boolean denoting whether a gene has other paralogous genes in the same set
    paralogs - a list of integers denoting the index of the other genes
    """
    def __init__(self,gene):
        """Gene(gene)

        A constructor that uses the dictionary loaded by scipy.io.loadmat
        """
        #assert(gene.__dict__.keys()==['is_alt','name','exons','transcript_valid','has_paralog',\
        #                              'terminal_info','paralogs',\
        #                              'stop','is_cDNA','has_cDNA','start','chr','is_valid',\
        #                              '_fieldnames','exons_confirmed','splicegraph',\
        #                              'is_alt_spliced','transcripts','strands'])
        self.name = gene.name
        self.chr = gene.chr
        self.start = gene.start
        self.stop = gene.stop
        self.transcripts = gene.transcripts
        self.exons = gene.exons
        self.exons_confirmed = gene.exons_confirmed
        self.strands = gene.strands
        self.terminal_info = gene.terminal_info
        self.is_cDNA = gene.is_cDNA
        self.has_cDNA = gene.has_cDNA
        self.transcript_valid = gene.transcript_valid
        self.is_valid = gene.is_valid
        self.splicegraph = SpliceGraph(gene.splicegraph)
        self.is_alt_spliced = gene.is_alt_spliced
        self.is_alt = gene.is_alt
        self.has_paralog = gene.has_paralog
        self.paralogs = gene.paralogs
        
    def __repr__(self):
        return 'name: ' + repr(self.name) + '\nchr: ' + repr(self.chr)\
               + '\nstart: ' + repr(self.start) + '\nstop: ' + repr(self.stop)\
               + '\ntranscripts: ' + repr(self.transcripts) + '\nexons: ' + repr(self.exons)\
               + '\nexons_confirmed: ' + repr(self.exons_confirmed) + '\nstrands: ' + repr(self.strands)\
               + '\nterminal_info: ' + repr(self.terminal_info) + '\nis_cDNA: ' + repr(self.is_cDNA)\
               + '\nhas_cDNA: ' + repr(self.has_cDNA) + '\ntranscript_valid: ' + repr(self.transcript_valid)\
               + '\nis_valid: ' + repr(self.is_valid) + '\nsplicegraph: ' + repr(self.splicegraph)\
               + 'is_alt_spliced: ' + repr(self.is_alt_spliced) + '\nis_alt: ' + repr(self.is_alt)\
               + '\nhas_paralog: ' + repr(self.has_paralog) + '\nparalogs: ' + repr(self.paralogs) + '\n'


    def get_longest_mrna(self,organism):
        """Get the mrna sequence of the isoform with the most nucleotides"""

        # Find all transcription starts
        vertices = self.splicegraph.exons
        edges = self.splicegraph.introns
        if vertices.ndim == 1:
            # there is only one exon, numpy stores it as a one dimensional array.
            mrna = load_genomic(self.chr,self.strands[0],vertices[0],vertices[1],organism)

        else:
            tstart = []
            tstop = []
            for ixv in xrange(len(vertices[0])):
                if sum(edges[ixv,:(ixv+1)]) == 0:
                    tstart.append(ixv)

            for ixv in xrange(len(vertices[0])):
                if sum(edges[(ixv+1):,ixv]) == 0:
                    tstop.append(ixv)

            # Get all isoforms
            isoforms = []
            if self.strands[0] == '+':
                for ixt in tstart:
                    isoform = []
                    dfs_visit(triu(edges),ixt,[],isoforms)
            else:
                for ixt in tstop:
                    isoform = []
                    dfs_visit(tril(edges),ixt,[],isoforms)
                for ixf in xrange(len(isoforms)):
                    isoforms[ixf].reverse()

            # Find longest and load the sequence.
            maxlen = 0
            maxix = 0
            for ixf in xrange(len(isoforms)):
                curlen = sum(vertices[1,isoforms[ixf]]-vertices[0,isoforms[ixf]])
                if curlen > maxlen:
                    maxlen = curlen
                    maxix = ixf

            mrna = load_genomic(self.chr,self.strands[0],\
                                vertices[0,isoforms[maxix]],vertices[1,isoforms[maxix]],organism)

        return mrna

    def get_dna(self,organism):
        """load the dna sequence covered by the gene"""
        return load_genomic(self.chr,self.strands[0],self.start,self.stop,organism)


def load_matfile(matfilename):
    """
    Load an array of genes from a matlab file.
    """
    print 'Loading ' + matfilename
    try:
        temp = scipy.io.loadmat(matfilename)
    except IOError:
        sys.stderr.write('File ' + matfilename + 'not found\n')
        sys.exit(2)

    print 'Parsing structure'
    matgenes = temp['genes']
    genes = []
    for ix,gene in enumerate(matgenes):
        if ix % 1000 == 0:
            print ix
        genes.append(Gene(gene))

    return genes



def export_mrna_fasta(filename,genes,linelen=60,organism='volvox1',condition=None):
    """export all the sequences of the longest mrna to a fasta file.

    One fasta entry per gene.
    """
    try:
        f = open(filename,'w')
    except IOError:
        sys.stderr.write('File ' + filename + ' cannot be written\n')

    for ix,g in enumerate(genes):
        if ix % 1000 == 0:
            print ix
        if (condition is None) or (eval('g.'+condition)):
            f.write('> Cluster '+' '.join([str(ix+1),g.chr,g.strands[0]])+' longest mrna sequence\n')
            s = g.get_longest_mrna(organism)
            for i in xrange(0, len(s), linelen):
                f.write(s[i:i+linelen] + '\n')
        
def export_dna_fasta(filename,genes,linelen=60,organism='volvox1',condition=None):
    """export all the sequences of the genes dna to a fasta file.

    One fasta entry per gene.
    """
    try:
        f = open(filename,'w')
    except IOError:
        sys.stderr.write('File ' + filename + ' cannot be written\n')

    for ix,g in enumerate(genes):
        if ix % 1000 == 0:
            print ix
        if (condition is None) or (eval('g.'+condition)):
            f.write('> Cluster '+' '.join([str(ix+1),g.chr,g.strands[0]])+' dna sequence\n')
            s = g.get_dna(organism)
            for i in xrange(0, len(s), linelen):
                f.write(s[i:i+linelen] + '\n')
        

def example():
    """Load a matlab file, and save it as a pickle file"""
    import gene
    from io_pickle import load, save

    genes = gene.load_matfile('confirmed_sequences.mat')
    save('confirmed_sequences.pbz',genes)
    genes2 = load('confirmed_sequences.pbz')
    
